"""
  Pool Test cases like start, stop and kill the device server, checking for
  emptyness, checking creation of elements with wrong arguments
"""

import os
import unittest
import poolunittest
import HTMLTestRunner
import time
import PyTango


class EmptyPoolTestCase(poolunittest.PoolTestCase):
    """Start, Stop, Kill and check emptyness of Device Pool"""

    EmptyStartStopStates = [PyTango.DevState.ON, PyTango.DevState.UNKNOWN]

    def setUp(self):
        poolunittest.PoolTestCase.setUp(self)
        self.pool_state_cb = poolunittest.DefaultTangoEventCallBack()
        self.pool_state_cb_id = self.pool_dp.subscribe_event("state",
                                                             PyTango.EventType.CHANGE,
                                                             self.pool_state_cb, [])

    def tearDown(self):
        if not self.pool_state_cb_id is None:
            try:
                self.pool_dp.unsubscribe_event(self.pool_state_cb_id)
                self.pool_state_cb_id = None
            except:
                pass
        self.pool_state_cb = None
        poolunittest.PoolTestCase.tearDown(self)

    def testStartStopWithKill(self):
        """Start & stop(kill) an empty Device Pool"""

        # Pool was started by setUp

        self.assertEqual(self.pool_dp.state(), PyTango.DevState.ON)

        self.stopPool()

        evt_values = self.pool_state_cb.getEventValues()
        self.assertEqual(evt_values, self.EmptyStartStopStates,
                         "State Event list(%s) does not match expected" % evt_values)

    def testStartStopWithAdminKill(self):
        """Start & stop(admin device kill) an empty Device Pool"""

        # Pool was started by setUp

        self.assertEqual(self.pool_dp.state(), PyTango.DevState.ON)

        self.pool_admin_dp = PyTango.DeviceProxy(
            "dserver/Pool/%s" % self.pool_ds_instance)

        self.pool_admin_dp.command_inout("Kill")

        self.waitPoolStop()

        evt_values = self.pool_state_cb.getEventValues()
        self.assertEqual(evt_values, self.EmptyStartStopStates,
                         "State Event list(%s) does not match expected" % evt_values)

    def testPing(self):
        """Empty Device Pool ping()"""

        ret = self.pool_dp.ping()
        self.assert_(ret > 0, "ping returned %d" % ret)

    def testLists(self):
        """Empty pool 'XList' attributes"""
        lst_attrs = [attr.name for attr in self.pool_dp.attribute_list_query_ex()
                     if attr.name.endswith('List') and attr.name.find("ControllerClass") == -1]

        for attr in lst_attrs:
            self.check_empty_attribute(self.pool_dp, attr)


class ControllerTestCase(poolunittest.PoolTestCase):
    """Test ControllerClassList, wrong controller creation"""

    def testCtrlClasslist(self):
        """ControllerClassList attribute"""

        ctrls = self.pool_dp.read_attribute("ControllerClasslist")

        v = ctrls.value

        len_v = len(v)

        self.assertNotEqual(len_v, 0, "Controller class list is empty")

        self.assert_(len_v >= 13, "Insufficient number of Controller classes found."
                     "Found %d. Should have at least 5")

        pool_path = self.getPoolPath()
        test_path = pool_path[1]
        cpp_path = pool_path[2]
        py_path = pool_path[3]

        ctrl = "Type: Motor - Class: SimuMotorController - File: %s/SimuMotorCtrl.la" % cpp_path
        self.assert_(ctrl in v, "SimuMotor controller not found")

        ctrl = "Type: CounterTimer - Class: UnixTimer - File: %s/UxTimerCtrl.la" % cpp_path
        self.assert_(ctrl in v, "Unix Timer controller not found")

        ctrl = "Type: CounterTimer - Class: SimuCoTiController - File: %s/SimuCTCtrl.py" % py_path
        self.assert_(ctrl in v, "SimuCoTi controller not found")

        ctrl = "Type: ZeroDExpChannel - Class: SimuZeroDController - File: %s/SimuZeroDCtrl.la" % cpp_path
        self.assert_(ctrl in v, "Simu0D controller not found")

        ctrl = "Type: PseudoMotor - Class: Slit - File: %s/PseudoMotorLib.py" % py_path
        self.assert_(ctrl in v, "Pseudo Motor controller not found")

        ctrl = "Type: Communication - Class: DummyCommunicationController - File: %s/DummyCommunicationController.py" % py_path
        self.assert_(ctrl in v, "DummyCommunication controller not found")

        ctrl = "Type: Motor - Class: FirePapController - File: %s/FirePapCtrl.py" % test_path
        self.assert_(ctrl in v, "FirePap controller not found")

        ctrl = "Type: Motor - Class: IcePapController - File: %s/WaterPapCtrl.py" % test_path
        self.assert_(ctrl in v, "IcePap controller not found")

        ctrl = "Type: Motor - Class: IcePapController - File: %s/WaterPapCtrl_init.py" % test_path
        self.assert_(ctrl in v, "IcePap controller not found")

        ctrl = "Type: Motor - Class: IcePapController - File: %s/WaterPapCtrl_mis_extra.py" % test_path
        self.assert_(ctrl in v, "IcePap controller not found")

        ctrl = "Type: Motor - Class: IcePapController - File: %s/WaterPapCtrl_mis_feat.py" % test_path
        self.assert_(ctrl in v, "IcePap controller not found")

        ctrl = "Type: Motor - Class: IcePapController - File: %s/WaterPapCtrl_prop.py" % test_path
        self.assert_(ctrl in v, "IcePap controller not found")

        ctrl = "Type: Motor - Class: IcePapController - File: %s/WaterPapCtrl_stat1.py" % test_path
        self.assert_(ctrl in v, "IcePap controller not found")

        ctrl = "Type: Motor - Class: IcePapController - File: %s/WaterPapCtrl_stat2.py" % test_path
        self.assert_(ctrl in v, "IcePap controller not found")

        ctrl = "Type: ZeroDExpChannel - Class: ElecMeterController - File: %s/ElecMeter.py" % test_path
        self.assert_(ctrl in v, "ElecMeter controller not found")

        ctrl = "Type: ZeroDExpChannel - Class: ElecMeterController - File: %s/ElecMeter_init.py" % test_path
        self.assert_(ctrl in v, "ElecMeter controller not found")


class MotorControllerTestCase(poolunittest.PoolTestCase):
    """Test motor controller creation, deletion, listing"""

    def needsMotorSimulator(self):
        return True

    def testCreateMotorControllerWrongArguments(self):
        """Create Motor controller with wrong arguments"""

        p = self.pool_dp
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["a", "b", "c"], "Pool_WrongArgumentNumber")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Motor", "a/b", "ctrl", "inst"], "Pool_FileUnsupportedType")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Motor", "a/b.truc", "ctrl", "inst"], "Pool_FileUnsupportedType")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Motor", "a", "b", "x"], "Pool_FileUnsupportedType")

        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Motor", "a.py", "b", "x"], "Pool_ControllerNotFound")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Motor", "a.la", "b", "x"], "Pool_ControllerNotFound")

        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Motor", "WaterPapCtrl.py", "b", "x"],
                            "Pool_CantLocatePythonClass")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Motor", "WaterPapCtrl_mis.py",
                                "IcePapController", "x"],
                            "Pool_PythonMethodNotFound")

        self.wrong_argument(p, "CreateController",
                            ["Motor", "FirePapCtrl.py",
                                "FirePapController", "third"],
                            "Pool_MissingPropertyValue")

    def testCreateDeleteMotorController(self):
        """Create/Delete Motor controller"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Motor", "WaterPapCtrl.py", "IcePapController", "first_mot_ctrl"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_mot_ctrl (WaterPapCtrl.IcePapController/first_mot_ctrl) - Motor Python ctrl (WaterPapCtrl.py)")

        p.command_inout("DeleteController", "first_mot_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testDeleteInexistingMotorController(self):
        """Try to delete inexisting motor controller"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Motor", "WaterPapCtrl.py", "IcePapController", "first_mot_ctrl"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_mot_ctrl (WaterPapCtrl.IcePapController/first_mot_ctrl) - Motor Python ctrl (WaterPapCtrl.py)")

        self.wrong_argument(p, "DeleteController", "Aaaa",
                            "Pool_ControllerNotFound")

        p.command_inout("DeleteController", "first_mot_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testDeleteMotorControllerWithMotors(self):
        """Try to delete a motor controller with motors"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Motor", "WaterPapCtrl.py", "IcePapController", "first_mot_ctrl"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_mot_ctrl (WaterPapCtrl.IcePapController/first_mot_ctrl) - Motor Python ctrl (WaterPapCtrl.py)")

        p.command_inout("CreateMotor", ([0], ["first_mot", "first_mot_ctrl"]))

        self.wrong_argument(p, "DeleteController", "first_mot_ctrl",
                            "Pool_CantDeleteController")

        p.command_inout("DeleteMotor", "first_mot")

        p.command_inout("DeleteController", "first_mot_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testCreateSameMotorControllerTwice(self):
        """Try to create same motor controller twice"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Motor", "WaterPapCtrl.py", "IcePapController", "first_mot_ctrl"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_mot_ctrl (WaterPapCtrl.IcePapController/first_mot_ctrl) - Motor Python ctrl (WaterPapCtrl.py)")

        self.wrong_argument(p, "CreateController",
                            ["Motor", "WaterPapCtrl.py",
                                "IcePapController", "first_mot_ctrl"],
                            "Pool_ControllerAlreadyCreated")

        p.command_inout("DeleteController", "first_mot_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testCreateDeleteTwoMotorControllers(self):
        """Create/Delete two motor controllers"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Motor", "WaterPapCtrl.py", "IcePapController", "first_mot_ctrl"])
        p.command_inout("CreateController",
                        ["Motor", "WaterPapCtrl.py", "IcePapController", "second_mot_ctrl"])

        # Check controller list attribute
        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 2)
        self.assertEqual(ctrl_list.value[0],
                         "first_mot_ctrl (WaterPapCtrl.IcePapController/first_mot_ctrl) - Motor Python ctrl (WaterPapCtrl.py)")
        self.assertEqual(ctrl_list.value[1],
                         "second_mot_ctrl (WaterPapCtrl.IcePapController/second_mot_ctrl) - Motor Python ctrl (WaterPapCtrl.py)")

        # Delete one controller and check controller list attribute
        p.command_inout("DeleteController", "second_mot_ctrl")
        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_mot_ctrl (WaterPapCtrl.IcePapController/first_mot_ctrl) - Motor Python ctrl (WaterPapCtrl.py)")

        p.command_inout("DeleteController", "first_mot_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testCreateMotorControllersWithMaxDevice(self):
        """Create a motor controller defining its mandatory MaxDevice property """

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Motor", "FirePapCtrl.py", "FirePapController",
                         "third", "MaxDevice", "8"])
        p.command_inout("DeleteController", "third")
        self.check_empty_attribute(p, "ControllerList")


class CounterTimerControllerTestCase(poolunittest.PoolTestCase):
    """Test CounterTimer controller creation, deletion, listing"""

    # The CounterTimer simulator needs the motor simulator to work so the motor
    # simulator becomes a requirement
    def needsMotorSimulator(self):
        return True

    def needsCounterTimerSimulator(self):
        return True

    def testCreateCounterTimerControllerWrongArguments(self):
        """Create CounterTimer controller with wrong arguments"""

        p = self.pool_dp
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["a", "b", "c"], "Pool_WrongArgumentNumber")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["CounterTimer", "a/b", "ctrl", "inst"], "Pool_FileUnsupportedType")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["CounterTimer", "a/b.truc", "ctrl", "inst"], "Pool_FileUnsupportedType")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["CounterTimer", "a", "b", "x"], "Pool_FileUnsupportedType")

        self.wrong_argument(self.pool_dp, "CreateController",
                            ["CounterTimer", "a.py", "b", "x"], "Pool_ControllerNotFound")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["CounterTimer", "a.la", "b", "x"], "Pool_ControllerNotFound")

        self.wrong_argument(self.pool_dp, "CreateController",
                            ["CounterTimer", "Vct6Ctrl.py", "b", "x"],
                            "Pool_CantLocatePythonClass")

        self.wrong_argument(p, "CreateController",
                            ["CounterTimer", "Vct6Ctrl.py", "Vct6Controller",
                             "first_ct_ctrl"],
                            "Pool_MissingPropertyValue")

    def testCreateDeleteCounterTimerController(self):
        """Create/Delete CounterTimer controller"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["CounterTimer", "Vct6Ctrl.py", "Vct6Controller",
                         "first_ct_ctrl", "CtrlDevName",
                         self.ctsim_ctrl_dev_name])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_ct_ctrl (Vct6Ctrl.Vct6Controller/first_ct_ctrl) - CounterTimer Python ctrl (Vct6Ctrl.py)")

        p.command_inout("DeleteController", "first_ct_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testDeleteInexistingCounterTimerController(self):
        """Try to delete inexisting CounterTimer controller"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["CounterTimer", "Vct6Ctrl.py", "Vct6Controller",
                         "first_ct_ctrl", "CtrlDevName",
                         self.ctsim_ctrl_dev_name])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_ct_ctrl (Vct6Ctrl.Vct6Controller/first_ct_ctrl) - CounterTimer Python ctrl (Vct6Ctrl.py)")

        self.wrong_argument(p, "DeleteController", "Aaaa",
                            "Pool_ControllerNotFound")

        p.command_inout("DeleteController", "first_ct_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testDeleteCounterTimerControllerWithCounters(self):
        """Try to delete a CounterTimer controller with counters"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["CounterTimer", "Vct6Ctrl.py", "Vct6Controller",
                         "first_ct_ctrl", "CtrlDevName",
                         self.ctsim_ctrl_dev_name])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_ct_ctrl (Vct6Ctrl.Vct6Controller/first_ct_ctrl) - CounterTimer Python ctrl (Vct6Ctrl.py)")

        p.command_inout("CreateExpChannel",
                        ([0], ["first_ct", "first_ct_ctrl"]))

        self.wrong_argument(p, "DeleteController", "first_ct_ctrl",
                            "Pool_CantDeleteController")

        p.command_inout("DeleteExpChannel", "first_ct")

        p.command_inout("DeleteController", "first_ct_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testCreateSameCounterTimerControllerTwice(self):
        """Try to create same CounterTimer controller twice"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["CounterTimer", "Vct6Ctrl.py", "Vct6Controller",
                         "first_ct_ctrl", "CtrlDevName",
                         self.ctsim_ctrl_dev_name])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_ct_ctrl (Vct6Ctrl.Vct6Controller/first_ct_ctrl) - CounterTimer Python ctrl (Vct6Ctrl.py)")

        self.wrong_argument(p, "CreateController",
                            ["CounterTimer", "Vct6Ctrl.py", "Vct6Controller",
                             "first_ct_ctrl", "CtrlDevName",
                             self.ctsim_ctrl_dev_name],
                            "Pool_ControllerAlreadyCreated")

        p.command_inout("DeleteController", "first_ct_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testCreateDeleteTwoCounterTimerControllers(self):
        """Create/Delete two CounterTimer controllers"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["CounterTimer", "Vct6Ctrl.py", "Vct6Controller",
                         "first_ct_ctrl", "CtrlDevName",
                         self.ctsim_ctrl_dev_name])
        p.command_inout("CreateController",
                        ["CounterTimer", "Vct6Ctrl.py", "Vct6Controller",
                         "second_ct_ctrl", "CtrlDevName",
                         self.ctsim_ctrl_dev_name])

        # Check controller list attribute
        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 2)
        self.assertEqual(ctrl_list.value[0],
                         "first_ct_ctrl (Vct6Ctrl.Vct6Controller/first_ct_ctrl) - CounterTimer Python ctrl (Vct6Ctrl.py)")
        self.assertEqual(ctrl_list.value[1],
                         "second_ct_ctrl (Vct6Ctrl.Vct6Controller/second_ct_ctrl) - CounterTimer Python ctrl (Vct6Ctrl.py)")

        # Delete one controller and check controller list attribute
        p.command_inout("DeleteController", "second_ct_ctrl")
        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_ct_ctrl (Vct6Ctrl.Vct6Controller/first_ct_ctrl) - CounterTimer Python ctrl (Vct6Ctrl.py)")

        self.pool_dp.command_inout("DeleteController", "first_ct_ctrl")
        self.check_empty_attribute(p, "ControllerList")


class ZeroDControllerTestCase(poolunittest.PoolTestCase):
    """Test 0D controller creation, deletion, listing"""

    def testCreate0DControllerWrongArguments(self):
        """Create 0D controller with wrong arguments"""

        p = self.pool_dp
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["a", "b", "c"], "Pool_WrongArgumentNumber")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["ZeroDExpChannel", "a/b", "ctrl", "inst"], "Pool_FileUnsupportedType")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["ZeroDExpChannel", "a/b.truc", "ctrl", "inst"], "Pool_FileUnsupportedType")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["ZeroDExpChannel", "a", "b", "x"], "Pool_FileUnsupportedType")

        self.wrong_argument(self.pool_dp, "CreateController",
                            ["ZeroDExpChannel", "a.py", "b", "x"], "Pool_ControllerNotFound")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["ZeroDExpChannel", "a.la", "b", "x"], "Pool_ControllerNotFound")

        self.wrong_argument(self.pool_dp, "CreateController",
                            ["ZeroDExpChannel", "ElecMeter.py", "b", "x"],
                            "Pool_CantLocatePythonClass")

    def testCreateDelete0DController(self):
        """Create/Delete 0D controller"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["ZeroDExpChannel", "ElecMeter.py",
                         "ElecMeterController", "first_0d_ctrl"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_0d_ctrl (ElecMeter.ElecMeterController/first_0d_ctrl) - ZeroDExpChannel Python ctrl (ElecMeter.py)")

        p.command_inout("DeleteController", "first_0d_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testDeleteInexisting0DController(self):
        """Try to delete inexisting 0D controller"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["ZeroDExpChannel", "ElecMeter.py",
                         "ElecMeterController", "first_0d_ctrl"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_0d_ctrl (ElecMeter.ElecMeterController/first_0d_ctrl) - ZeroDExpChannel Python ctrl (ElecMeter.py)")

        self.wrong_argument(p, "DeleteController", "Aaaa",
                            "Pool_ControllerNotFound")

        p.command_inout("DeleteController", "first_0d_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testDelete0DControllerWithChannels(self):
        """Try to delete a 0D controller with channels"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["ZeroDExpChannel", "ElecMeter.py",
                         "ElecMeterController", "first_0d_ctrl"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_0d_ctrl (ElecMeter.ElecMeterController/first_0d_ctrl) - ZeroDExpChannel Python ctrl (ElecMeter.py)")

        p.command_inout("CreateExpChannel",
                        ([0], ["first_0d", "first_0d_ctrl"]))

        self.wrong_argument(p, "DeleteController", "first_0d_ctrl",
                            "Pool_CantDeleteController")

        p.command_inout("DeleteExpChannel", "first_0d")

        p.command_inout("DeleteController", "first_0d_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testCreateSame0DControllerTwice(self):
        """Try create same 0D controller twice"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["ZeroDExpChannel", "ElecMeter.py",
                         "ElecMeterController", "first_0d_ctrl"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_0d_ctrl (ElecMeter.ElecMeterController/first_0d_ctrl) - ZeroDExpChannel Python ctrl (ElecMeter.py)")

        self.wrong_argument(p, "CreateController",
                            ["ZeroDExpChannel", "ElecMeter.py",
                             "ElecMeterController", "first_0d_ctrl"],
                            "Pool_ControllerAlreadyCreated")

        p.command_inout("DeleteController", "first_0d_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testCreateDeleteTwo0DControllers(self):
        """Create/Delete two 0D controllers"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["ZeroDExpChannel", "ElecMeter.py",
                         "ElecMeterController", "first_0d_ctrl"])
        p = self.pool_dp
        p.command_inout("CreateController",
                        ["ZeroDExpChannel", "ElecMeter.py",
                         "ElecMeterController", "second_0d_ctrl"])

        # Check controller list attribute
        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 2)
        self.assertEqual(ctrl_list.value[0],
                         "first_0d_ctrl (ElecMeter.ElecMeterController/first_0d_ctrl) - ZeroDExpChannel Python ctrl (ElecMeter.py)")
        self.assertEqual(ctrl_list.value[1],
                         "second_0d_ctrl (ElecMeter.ElecMeterController/second_0d_ctrl) - ZeroDExpChannel Python ctrl (ElecMeter.py)")

        # Delete one controller and check controller list attribute
        p.command_inout("DeleteController", "second_0d_ctrl")
        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_0d_ctrl (ElecMeter.ElecMeterController/first_0d_ctrl) - ZeroDExpChannel Python ctrl (ElecMeter.py)")

        self.pool_dp.command_inout("DeleteController", "first_0d_ctrl")
        self.check_empty_attribute(p, "ControllerList")


class CommunicationControllerTestCase(poolunittest.PoolTestCase):
    """Test communication controller creation, deletion, listing"""

    def testCreateCommunicationControllerWrongArguments(self):
        """Create Communication controller with wrong arguments"""

        p = self.pool_dp
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["a", "b", "c"], "Pool_WrongArgumentNumber")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Communication", "a/b", "ctrl", "inst"], "Pool_FileUnsupportedType")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Communication", "a/b.truc", "ctrl", "inst"], "Pool_FileUnsupportedType")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Communication", "a", "b", "x"], "Pool_FileUnsupportedType")

        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Communication", "a.py", "b", "x"], "Pool_ControllerNotFound")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Communication", "a.la", "b", "x"], "Pool_ControllerNotFound")

        self.wrong_argument(self.pool_dp, "CreateController",
                            ["Communication", "DummyCommunicationController.py", "b", "x"],
                            "Pool_CantLocatePythonClass")

    def testCreateDeleteCommunicationController(self):
        """Create/Delete Communication controller"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Communication", "DummyCommunicationController.py",
                         "DummyCommunicationController", "first_com_ctrl"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_com_ctrl (DummyCommunicationController.DummyCommunicationController/first_com_ctrl) - Communication Python ctrl (DummyCommunicationController.py)")

        p.command_inout("DeleteController", "first_com_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testDeleteInexistingCommunicationController(self):
        """Try to delete inexisting Communication controller"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Communication", "DummyCommunicationController.py",
                         "DummyCommunicationController", "first_com_ctrl"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_com_ctrl (DummyCommunicationController.DummyCommunicationController/first_com_ctrl) - Communication Python ctrl (DummyCommunicationController.py)")

        self.wrong_argument(p, "DeleteController", "Aaaa",
                            "Pool_ControllerNotFound")

        p.command_inout("DeleteController", "first_com_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testDeleteCommunicationControllerWithChannels(self):
        """Try to delete a Communication controller with channels"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Communication", "DummyCommunicationController.py",
                         "DummyCommunicationController", "first_com_ctrl"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_com_ctrl (DummyCommunicationController.DummyCommunicationController/first_com_ctrl) - Communication Python ctrl (DummyCommunicationController.py)")

        p.command_inout("CreateComChannel",
                        ([0], ["first_comch", "first_com_ctrl"]))

        self.wrong_argument(p, "DeleteController", "first_com_ctrl",
                            "Pool_CantDeleteController")

        p.command_inout("DeleteComChannel", "first_comch")

        p.command_inout("DeleteController", "first_com_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testCreateSameCommunicationControllerTwice(self):
        """Try create same Communication controller twice"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Communication", "DummyCommunicationController.py",
                         "DummyCommunicationController", "first_com_ctrl"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_com_ctrl (DummyCommunicationController.DummyCommunicationController/first_com_ctrl) - Communication Python ctrl (DummyCommunicationController.py)")

        self.wrong_argument(p, "CreateController",
                            ["Communication", "DummyCommunicationController.py",
                             "DummyCommunicationController", "first_com_ctrl"],
                            "Pool_ControllerAlreadyCreated")

        p.command_inout("DeleteController", "first_com_ctrl")
        self.check_empty_attribute(p, "ControllerList")

    def testCreateDeleteTwoCommunicationControllers(self):
        """Create/Delete two Communication controllers"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Communication", "DummyCommunicationController.py",
                         "DummyCommunicationController", "first_com_ctrl"])
        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Communication", "DummyCommunicationController.py",
                         "DummyCommunicationController", "second_com_ctrl"])

        # Check controller list attribute
        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 2)
        self.assertEqual(ctrl_list.value[0],
                         "first_com_ctrl (DummyCommunicationController.DummyCommunicationController/first_com_ctrl) - Communication Python ctrl (DummyCommunicationController.py)")
        self.assertEqual(ctrl_list.value[1],
                         "second_com_ctrl (DummyCommunicationController.DummyCommunicationController/second_com_ctrl) - Communication Python ctrl (DummyCommunicationController.py)")

        # Delete one controller and check controller list attribute
        p.command_inout("DeleteController", "second_com_ctrl")
        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_com_ctrl (DummyCommunicationController.DummyCommunicationController/first_com_ctrl) - Communication Python ctrl (DummyCommunicationController.py)")

        self.pool_dp.command_inout("DeleteController", "first_com_ctrl")
        self.check_empty_attribute(p, "ControllerList")


class PseudoMotorControllerTestCase(poolunittest.PoolTestCase):
    """Test PseudoMotor controller creation, deletion, listing"""

    def needsMotorSimulator(self):
        return True

    def setUp(self):
        poolunittest.PoolTestCase.setUp(self)
        p = self.pool_dp
        p.command_inout("CreateController",
                        ["Motor", "SimuMotorCtrl.la", "SimuMotorController",
                         "first_simumot_ctrl",
                         "DevName", self.motsim_ctrl_dev_name])

        p.command_inout(
            "CreateMotor", ([0], ["first_simu_mot", "first_simumot_ctrl"]))
        p.command_inout(
            "CreateMotor", ([1], ["second_simu_mot", "first_simumot_ctrl"]))

    def tearDown(self):
        p = self.pool_dp

        p.command_inout("DeleteMotor", "first_simu_mot")
        p.command_inout("DeleteMotor", "second_simu_mot")
        p.command_inout("DeleteController", "first_simumot_ctrl")
        poolunittest.PoolTestCase.tearDown(self)

    def testCreatePseudoMotorControllerWrongArguments(self):
        """Create Pseudo Motor controller with wrong arguments"""

        p = self.pool_dp
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["a", "b", "c"], "Pool_WrongArgumentNumber")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["PseudoMotor", "a/b", "ctrl", "inst"], "Pool_FileUnsupportedType")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["PseudoMotor", "a/b.truc", "ctrl", "inst"], "Pool_FileUnsupportedType")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["PseudoMotor", "a", "b", "x"], "Pool_FileUnsupportedType")

        self.wrong_argument(self.pool_dp, "CreateController",
                            ["PseudoMotor", "a.py", "b", "x"], "Pool_ControllerNotFound")
        self.wrong_argument(self.pool_dp, "CreateController",
                            ["PseudoMotor", "a.la", "b", "x"], "Pool_ControllerNotFound")

        self.wrong_argument(self.pool_dp, "CreateController",
                            ["PseudoMotor", "PseudoMotorLib.py", "b", "x"],
                            "Pool_CantLocatePythonClass")

        self.wrong_argument(self.pool_dp, "CreateController",
                            ["PseudoMotor", "PseudoMotorLib.py", "b", "x"],
                            "Pool_CantLocatePythonClass")

        self.wrong_argument(p, "CreateController",
                            ["PseudoMotor", "PseudoMotorLib.py", "Slit",
                             "first_slit_ctrl",
                             "inexisting_simu_mot", "second_simu_mot",
                             "first_gap_mot", "first_offset_mot"],
                            "Pool_MotorNotDefined")

        self.wrong_argument(p, "CreateController",
                            ["PseudoMotor", "PseudoMotorLib.py", "Slit",
                             "first_slit_ctrl",
                             "first_simu_mot", "second_simu_mot",
                             "first_gap_mot"],
                            "Pool_WrongArgumentNumber")

    def testCreateDeletePseudoMotorController(self):
        """Create/Delete PseudoMotor controller"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["PseudoMotor", "PseudoMotorLib.py",
                         "Slit", "first_slit_ctrl",
                         "first_simu_mot", "second_simu_mot",
                         "first_gap_mot", "first_offset_mot"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 2)
        self.assertEqual(ctrl_list.value[0],
                         "first_simumot_ctrl (SimuMotorCtrl.SimuMotorController/first_simumot_ctrl) - Motor Cpp ctrl (SimuMotorCtrl.la)")
        self.assertEqual(ctrl_list.value[1],
                         "first_slit_ctrl (PseudoMotorLib.Slit/first_slit_ctrl) - PseudoMotor Python ctrl (PseudoMotorLib.py)")

        p.command_inout("DeletePseudoMotor", "first_gap_mot")
        p.command_inout("DeletePseudoMotor", "first_offset_mot")

        p.command_inout("DeleteController", "first_slit_ctrl")

        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_simumot_ctrl (SimuMotorCtrl.SimuMotorController/first_simumot_ctrl) - Motor Cpp ctrl (SimuMotorCtrl.la)")

    def testDeleteInexistingPseudoMotorController(self):
        """Try to delete inexisting PseudoMotor controller"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["PseudoMotor", "PseudoMotorLib.py",
                         "Slit", "first_slit_ctrl",
                         "first_simu_mot", "second_simu_mot",
                         "first_gap_mot", "first_offset_mot"])

        self.wrong_argument(p, "DeleteController", "Aaaa",
                            "Pool_ControllerNotFound")

        p.command_inout("DeletePseudoMotor", "first_gap_mot")
        p.command_inout("DeletePseudoMotor", "first_offset_mot")

        p.command_inout("DeleteController", "first_slit_ctrl")
        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_simumot_ctrl (SimuMotorCtrl.SimuMotorController/first_simumot_ctrl) - Motor Cpp ctrl (SimuMotorCtrl.la)")

    def testDeletePseudoMotorControllerWithPseudoMotors(self):
        """Try to delete a PseudoMotor controller with pseudo motors"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["PseudoMotor", "PseudoMotorLib.py",
                         "Slit", "first_slit_ctrl",
                         "first_simu_mot", "second_simu_mot",
                         "first_gap_mot", "first_offset_mot"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 2)
        self.assertEqual(ctrl_list.value[0],
                         "first_simumot_ctrl (SimuMotorCtrl.SimuMotorController/first_simumot_ctrl) - Motor Cpp ctrl (SimuMotorCtrl.la)")
        self.assertEqual(ctrl_list.value[1],
                         "first_slit_ctrl (PseudoMotorLib.Slit/first_slit_ctrl) - PseudoMotor Python ctrl (PseudoMotorLib.py)")

        self.wrong_argument(p, "DeleteController", "first_slit_ctrl",
                            "Pool_CantDeleteController")

        self.wrong_argument(p, "DeleteMotor", "first_simu_mot",
                            "Pool_CantDeleteMotor")
        self.wrong_argument(p, "DeleteMotor", "second_simu_mot",
                            "Pool_CantDeleteMotor")

        p.command_inout("DeletePseudoMotor", "first_gap_mot")
        p.command_inout("DeletePseudoMotor", "first_offset_mot")

        p.command_inout("DeleteController", "first_slit_ctrl")

        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_simumot_ctrl (SimuMotorCtrl.SimuMotorController/first_simumot_ctrl) - Motor Cpp ctrl (SimuMotorCtrl.la)")

    def testCreateSamePseudoMotorControllerTwice(self):
        """Try create same PseudoMotor controller twice"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["PseudoMotor", "PseudoMotorLib.py",
                         "Slit", "first_slit_ctrl",
                         "first_simu_mot", "second_simu_mot",
                         "first_gap_mot", "first_offset_mot"])

        ctrl_list = p.read_attribute("ControllerList")

        self.assertEqual(ctrl_list.dim_x, 2)
        self.assertEqual(ctrl_list.value[0],
                         "first_simumot_ctrl (SimuMotorCtrl.SimuMotorController/first_simumot_ctrl) - Motor Cpp ctrl (SimuMotorCtrl.la)")
        self.assertEqual(ctrl_list.value[1],
                         "first_slit_ctrl (PseudoMotorLib.Slit/first_slit_ctrl) - PseudoMotor Python ctrl (PseudoMotorLib.py)")

        self.wrong_argument(p, "CreateController",
                            ["PseudoMotor", "PseudoMotorLib.py",
                             "Slit", "first_slit_ctrl",
                             "first_simu_mot", "second_simu_mot",
                             "first_gap_mot", "first_offset_mot"],
                            "Pool_ControllerAlreadyCreated")

        p.command_inout("DeletePseudoMotor", "first_gap_mot")
        p.command_inout("DeletePseudoMotor", "first_offset_mot")

        p.command_inout("DeleteController", "first_slit_ctrl")

        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_simumot_ctrl (SimuMotorCtrl.SimuMotorController/first_simumot_ctrl) - Motor Cpp ctrl (SimuMotorCtrl.la)")

    def testCreateDeleteTwoPseudoMotorControllers(self):
        """Create/Delete two PseudoMotor controllers"""

        p = self.pool_dp
        p.command_inout("CreateController",
                        ["PseudoMotor", "PseudoMotorLib.py",
                         "Slit", "first_slit_ctrl",
                         "first_simu_mot", "second_simu_mot",
                         "first_gap_mot", "first_offset_mot"])

        #
        # Repeating pseudo motor names is an error
        #
        self.wrong_argument(p, "CreateController",
                            ["PseudoMotor", "PseudoMotorLib.py",
                             "Slit", "second_slit_ctrl",
                             "first_simu_mot", "second_simu_mot",
                             "first_gap_mot", "second_offset_mot"],
                            "Pool_PseudoMotorAlreadyCreated")

        p.command_inout("CreateController",
                        ["PseudoMotor", "PseudoMotorLib.py",
                         "Slit", "second_slit_ctrl",
                         "first_simu_mot", "second_simu_mot",
                         "second_gap_mot", "second_offset_mot"])

        # Check controller list attribute
        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 3)
        self.assertEqual(ctrl_list.value[0],
                         "first_simumot_ctrl (SimuMotorCtrl.SimuMotorController/first_simumot_ctrl) - Motor Cpp ctrl (SimuMotorCtrl.la)")
        self.assertEqual(ctrl_list.value[1],
                         "first_slit_ctrl (PseudoMotorLib.Slit/first_slit_ctrl) - PseudoMotor Python ctrl (PseudoMotorLib.py)")
        self.assertEqual(ctrl_list.value[2],
                         "second_slit_ctrl (PseudoMotorLib.Slit/second_slit_ctrl) - PseudoMotor Python ctrl (PseudoMotorLib.py)")

        # Delete one controller and check controller list attribute
        p.command_inout("DeletePseudoMotor", "second_gap_mot")
        p.command_inout("DeletePseudoMotor", "second_offset_mot")

        p.command_inout("DeleteController", "second_slit_ctrl")
        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 2)
        self.assertEqual(ctrl_list.value[0],
                         "first_simumot_ctrl (SimuMotorCtrl.SimuMotorController/first_simumot_ctrl) - Motor Cpp ctrl (SimuMotorCtrl.la)")
        self.assertEqual(ctrl_list.value[1],
                         "first_slit_ctrl (PseudoMotorLib.Slit/first_slit_ctrl) - PseudoMotor Python ctrl (PseudoMotorLib.py)")

        p.command_inout("DeletePseudoMotor", "first_gap_mot")
        p.command_inout("DeletePseudoMotor", "first_offset_mot")

        self.pool_dp.command_inout("DeleteController", "first_slit_ctrl")
        ctrl_list = p.read_attribute("ControllerList")
        self.assertEqual(ctrl_list.dim_x, 1)
        self.assertEqual(ctrl_list.value[0],
                         "first_simumot_ctrl (SimuMotorCtrl.SimuMotorController/first_simumot_ctrl) - Motor Cpp ctrl (SimuMotorCtrl.la)")


def suite():
    s = poolunittest.PoolTestSuite()

    empty_pool_suite = unittest.makeSuite(EmptyPoolTestCase, 'test')
    ctrl_pool_suite = unittest.makeSuite(ControllerTestCase, 'test')
    mot_ctrl_pool_suite = unittest.makeSuite(MotorControllerTestCase, 'test')
    ct_ctrl_pool_suite = unittest.makeSuite(
        CounterTimerControllerTestCase, 'test')
    zd_ctrl_pool_suite = unittest.makeSuite(ZeroDControllerTestCase, 'test')
    com_ctrl_pool_suite = unittest.makeSuite(
        CommunicationControllerTestCase, 'test')
    pm_ctrl_pool_suite = unittest.makeSuite(
        PseudoMotorControllerTestCase, 'test')

    s.addTest(empty_pool_suite)
    s.addTest(ctrl_pool_suite)
    s.addTest(mot_ctrl_pool_suite)
    s.addTest(ct_ctrl_pool_suite)
    s.addTest(zd_ctrl_pool_suite)
    s.addTest(com_ctrl_pool_suite)
    s.addTest(pm_ctrl_pool_suite)

    return s


def m1():
    unittest.main()


def m2(test_output):
    test_output_file = open(test_output, "w")
    runner = poolunittest.PoolTestRunner(verbosity=2, stream=test_output_file)
    unittest.main(testRunner=runner, testLoader=poolunittest.PoolTestLoader())


def m3(test_output):
    test_output_file = file(test_output, 'wb')
    runner = HTMLTestRunner.HTMLTestRunner(
        stream=test_output_file,
        title='Device Pool Unit Test',
        description='Device Pool Unit Test package',
        verbosity=2)
    runner.run(suite())

if __name__ == "__main__":
    test_output = "test_result.html"
    m3(test_output)



##############################################################################
##
# This file is part of Sardana
##
# http://www.tango-controls.org/static/sardana/latest/doc/html/axisex.html
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

import time
import copy
import numpy

from sardana import State
from sardana.pool import AcqSynch
from sardana.pool.controller import OneDController, MaxDimSize
from sardana.pool.controller import DefaultValue, Description, FGet, FSet, Type


def gauss(x, mean, ymax, fwhm, yoffset=0):
    return yoffset + ymax * numpy.power(2, -4 * ((x - mean) / fwhm)**2)


class Channel:

    def __init__(self, idx):
        self.idx = idx            # 1 based axisex
        self.value = []
        self.is_counting = False
        self.active = True
        self.amplitude = BaseValue('1.0')
        self._counter = 0


class BaseValue(object):

    def __init__(self, value):
        self.raw_value = value
        self.init()

    def init(self):
        self.value = float(self.raw_value)

    def get(self):
        return self.value

    def get_value_name(self):
        return self.raw_value


class TangoValue(BaseValue):

    def init(self):
        import PyTango
        self.attr_proxy = PyTango.AttributeProxy(self.raw_value)

    def get(self):
        return self.attr_proxy.read().value


class DummyOneDController(OneDController):
    "This class is the Tango Sardana OneDController controller for tests"

    gender = "Simulation"
    model = "Basic"
    organization = "Sardana team"

    MaxDevice = 1024

    BufferSize = 1024,

    axis_attributes = {
        'Amplitude': {
            Type: str,
            FGet: 'getAmplitude',
            FSet: 'setAmplitude',
            Description: 'Amplitude. Maybe a number or a tango attribute(must start with tango://)',
            DefaultValue: '1.0'},
    }

    def __init__(self, inst, props, *args, **kwargs):
        OneDController.__init__(self, inst, props, *args, **kwargs)
        self._repetitions = 1
        self.channels = self.MaxDevice * [None, ]
        self.reset()

    def GetAxisAttributes(self, axis):
        # the default max shape for 'value' is (16*1024,). We don't need so much
        # so we set it to BufferSize
        attrs = super(DummyOneDController, self).GetAxisAttributes(axis)
        attrs['Value'][MaxDimSize] = self.BufferSize
        return attrs

    def reset(self):
        self.start_time = None
        self.integ_time = None
        self.monitor_count = None
        self.read_channels = {}
        self.counting_channels = {}

    def AddDevice(self, axis):
        idx = axis - 1
        self.channels[idx] = channel = Channel(axis)
        channel.value = numpy.zeros(self.BufferSize, dtype=numpy.float64)

    def DeleteDevice(self, axis):
        idx = axis - 1
        self.channels[idx] = None

    def PreStateAll(self):
        pass

    def PreStateOne(self, axis):
        pass

    def StateAll(self):
        pass

    def StateOne(self, axis):
        idx = axis - 1
        sta = State.On
        status = "Stopped"
        if axis in self.counting_channels:
            channel = self.channels[idx]
            now = time.time()
            elapsed_time = now - self.start_time
            self._updateChannelState(axis, elapsed_time)
            if channel.is_counting:
                sta = State.Moving
                status = "Acquiring"
        return sta, status

    def _updateChannelState(self, axis, elapsed_time):
        if self._synchronization == AcqSynch.SoftwareTrigger:
            if self.integ_time is not None:
                # counting in time
                if elapsed_time >= self.integ_time:
                    self._finish(elapsed_time)
            elif self.monitor_count is not None:
                # monitor counts
                v = int(elapsed_time * 100 * axis)
                if v >= self.monitor_count:
                    self._finish(elapsed_time)
        elif self._synchronization in (AcqSynch.HardwareTrigger,
                                       AcqSynch.HardwareGate):
            if self.integ_time is not None:
                if elapsed_time > self._repetitions * (self.integ_time +
                                                       self._latency_time):
                    self._finish(elapsed_time)

    def _updateChannelValue(self, axis, elapsed_time):
        channel = self.channels[axis - 1]
        t = elapsed_time
        if self._synchronization == AcqSynch.SoftwareTrigger:
            if self.integ_time is not None and not channel.is_counting:
                t = self.integ_time
            x = numpy.linspace(-10, 10, self.BufferSize[0])
            amplitude = axis * t * channel.amplitude.get()
            channel.value = gauss(x, 0, amplitude, 4)
        elif self._synchronization in (AcqSynch.HardwareTrigger,
                                       AcqSynch.HardwareGate):
            if self.integ_time is not None:
                n = int(t // self.integ_time)
                cp = 0
                if n > self._repetitions:
                    cp = n - self._repetitions
                n = n - channel._counter - cp
                t = self.integ_time
                x = numpy.linspace(-10, 10, self.BufferSize[0])
                amplitude = axis * t * channel.amplitude.get()
                value = gauss(x, 0, amplitude, 4)
                channel.buffer_values = [value] * n

    def _finish(self, elapsed_time, axis=None):
        if axis is None:
            for axis, channel in list(self.counting_channels.items()):
                channel.is_counting = False
                self._updateChannelValue(axis, elapsed_time)
        else:
            if axis in self.counting_channels:
                channel = self.counting_channels[axis]
                channel.is_counting = False
                self._updateChannelValue(axis, elapsed_time)
            else:
                channel = self.channels[axis - 1]
                channel.is_counting = False
        self.counting_channels = {}

    def PreReadAll(self):
        self.read_channels = {}

    def PreReadOne(self, axis):
        channel = self.channels[axis - 1]
        self.read_channels[axis] = channel

    def ReadAll(self):
        # if in acquisition then calculate the values to return
        if self.counting_channels:
            now = time.time()
            elapsed_time = now - self.start_time
            for axis, channel in list(self.read_channels.items()):
                self._updateChannelState(axis, elapsed_time)
                if channel.is_counting:
                    self._updateChannelValue(axis, elapsed_time)

    def ReadOne(self, axis):
        self._log.debug("ReadOne(%s)", axis)
        channel = self.read_channels[axis]
        if self._synchronization == AcqSynch.SoftwareTrigger:
            v = channel.value.tolist()
        elif self._synchronization in (AcqSynch.HardwareTrigger,
                                       AcqSynch.HardwareGate):
            v = copy.deepcopy(channel.buffer_values)
            channel._counter = channel._counter + len(v)
            channel.buffer_values.__init__()
        self._log.debug("DummyOneDController.ReadOne: returns %r" % v)
        return v

    def PreStartAll(self):
        self.counting_channels = {}

    def PreStartOne(self, axis, value):
        idx = axis - 1
        channel = self.channels[idx]
        channel.value = 0.0
        channel._counter = 0
        channel.buffer_values = []
        self.counting_channels[axis] = channel
        return True

    def StartOne(self, axis, value):
        self.counting_channels[axis].is_counting = True

    def StartAll(self):
        self.start_time = time.time()

    def LoadOne(self, axis, value, repetitions, latency_time):
        idx = axis - 1
        if value > 0:
            self.integ_time = value
            self.monitor_count = None
        else:
            self.integ_time = None
            self.monitor_count = -value
        self._repetitions = repetitions

    def AbortOne(self, axis):
        now = time.time()
        if axis in self.counting_channels:
            elapsed_time = now - self.start_time
            self._finish(elapsed_time, axis=axis)

    def getAmplitude(self, axis):
        idx = axis - 1
        channel = self.channels[idx]
        return channel.amplitude.get_value_name()

    def setAmplitude(self, axis, value):
        idx = axis - 1
        channel = self.channels[idx]

        klass = BaseValue
        if value.startswith("tango://"):
            klass = TangoValue
        channel.amplitude = klass(value)



#!/usr/bin/env python

##############################################################################
##
# This file is part of Sardana
##
# http://www.sardana-controls.org/
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

__all__ = ['createPoolController', 'createPoolCounterTimer',
           'createPoolZeroDExpChannel', 'createPoolTwoDExpChannel',
           'createPoolTriggerGate',
           'createPoolMotor', 'createPoolPseudoCounter',
           'createPoolPseudoMotor', 'createPoolMeasurementGroup',
           'createControllerConfiguration',
           'createTimerableControllerConfiguration',
           'createCTAcquisitionConfiguration',
           'createElemConf', 'createCtrlConf', 'createConfbyCtrlKlass',
           'createMGUserConfiguration']
import copy

from sardana.sardanadefs import ElementType
from sardana.pool.poolcontroller import PoolController,\
    PoolPseudoMotorController, PoolPseudoCounterController
from sardana.pool.poolcountertimer import PoolCounterTimer
from sardana.pool.poolzerodexpchannel import Pool0DExpChannel
from sardana.pool.pooltwodexpchannel import Pool2DExpChannel
from sardana.pool.pooltriggergate import PoolTriggerGate
from sardana.pool.poolmotor import PoolMotor
from sardana.pool.poolpseudocounter import PoolPseudoCounter
from sardana.pool.poolpseudomotor import PoolPseudoMotor
from sardana.pool.poolmeasurementgroup import PoolMeasurementGroup, \
    MeasurementConfiguration, ControllerConfiguration, ChannelConfiguration


def createPoolController(pool, conf):
    '''Method to create a PoolController using a configuration dictionary
    '''
    kwargs = copy.deepcopy(conf)

    ctrl_manager = pool.ctrl_manager
    ctrl_class_info = None
    ctrl_lib_info = ctrl_manager.getControllerLib(kwargs['library'])
    if ctrl_lib_info is not None:
        ctrl_class_info = ctrl_lib_info.get_controller(kwargs['klass'])
    main_type = ctrl_class_info.type_names[0]
    # check if all controller properties are present in conf.
    # in case of missing prop. and existing default value, use the default
    properties = kwargs['properties']
    if ctrl_class_info:
        ctrl_properties = ctrl_class_info.ctrl_properties
    else:
        ctrl_properties = {}
    for prop_info in list(ctrl_properties.values()):
        prop_name = prop_info.name
        prop_value = properties.get(prop_name)
        if prop_value is None:
            if prop_info.default_value is None:
                class_name = ctrl_class_info.get_name()
                raise Exception("Controller class '%s' needs property '%s'"
                                % (class_name, prop_name))
            properties[prop_name] = prop_info.default_value
    kwargs['pool'] = pool
    kwargs['lib_info'] = ctrl_lib_info
    kwargs['class_info'] = ctrl_class_info
    if main_type == "PseudoMotor":
        klass = PoolPseudoMotorController
    elif main_type == "PseudoCounter":
        klass = PoolPseudoCounterController
    else:
        klass = PoolController
    return klass(**kwargs)


def createPoolCounterTimer(pool, poolcontroller, conf):
    '''Method to create a PoolCounterTimer using a configuration dictionary
    '''
    kwargs = copy.deepcopy(conf)
    kwargs['pool'] = pool
    kwargs['ctrl'] = poolcontroller
    ct = PoolCounterTimer(**kwargs)
    poolcontroller.add_element(ct)
    return ct


def createPoolZeroDExpChannel(pool, poolcontroller, conf):
    '''Method to create a ZeroDExpChannel using a configuration dictionary
    '''
    kwargs = copy.deepcopy(conf)
    kwargs['pool'] = pool
    kwargs['ctrl'] = poolcontroller
    return Pool0DExpChannel(**kwargs)


def createPoolTwoDExpChannel(pool, poolcontroller, conf):
    '''Method to create a ZeroDExpChannel using a configuration dictionary
    '''
    kwargs = copy.deepcopy(conf)
    kwargs['pool'] = pool
    kwargs['ctrl'] = poolcontroller
    return Pool2DExpChannel(**kwargs)


def createPoolTriggerGate(pool, poolcontroller, conf):
    '''Method to create a PoolTriggerGate using a configuration dictionary
    '''
    kwargs = copy.deepcopy(conf)
    kwargs['pool'] = pool
    kwargs['ctrl'] = poolcontroller
    return PoolTriggerGate(**kwargs)


def createPoolMotor(pool, poolcontroller, conf):
    '''Method to create a PoolMotor using a configuration dictionary
    '''
    kwargs = copy.deepcopy(conf)
    kwargs['pool'] = pool
    kwargs['ctrl'] = poolcontroller
    return PoolMotor(**kwargs)


def createPoolPseudoCounter(pool, poolcontroller, conf, elements=()):
    '''Method to create a PoolPseudoCounter using a configuration dictionary
    '''
    kwargs = copy.deepcopy(conf)
    kwargs['pool'] = pool
    kwargs['ctrl'] = poolcontroller
    kwargs['user_elements'] = elements
    return PoolPseudoCounter(**kwargs)


def createPoolPseudoMotor(pool, poolcontroller, conf, elements=()):
    '''Method to create a PoolPseudoMotor using a configuration dictionary
    '''
    kwargs = copy.deepcopy(conf)
    kwargs['pool'] = pool
    kwargs['ctrl'] = poolcontroller
    kwargs['user_elements'] = elements
    return PoolPseudoMotor(**kwargs)


def createPoolMeasurementGroup(pool, conf):
    '''Method to create a PoolMeasurementGroup using a configuration dictionary
    '''
    kwargs = copy.deepcopy(conf)
    id = kwargs.get('id')
    if id is None:
        kwargs['id'] = pool.get_free_id()
    kwargs['pool'] = pool
    return PoolMeasurementGroup(**kwargs)


def createControllerConfiguration(pool_ctrl, pool_channels):
    conf_ctrl = ControllerConfiguration(pool_ctrl)
    for pool_channel in pool_channels:
        channel = ChannelConfiguration(pool_channel)
        channel.controller = conf_ctrl
        conf_ctrl.add_channel(channel)
    return conf_ctrl


def createTimerableControllerConfiguration(pool_ctrl, pool_channels):
    conf_ctrl = createControllerConfiguration(pool_ctrl, pool_channels)
    channel = conf_ctrl.get_channels(enabled=True)[0]
    conf_ctrl.timer = channel
    conf_ctrl.monitor = channel
    return conf_ctrl


def createCTAcquisitionConfiguration(ctrls, ctrl_channels):
    '''Method to create CTAcquisition configuration. Order of the sequences is
    important. For all sequences, the element of a given position refers
    the same controller.

    :param ctrls: sequence of the controllers used by the action
    :type ctrls: seq<sardana.pool.PoolController>
    :param ctrl_channels: sequence of the sequences of the channels
    corresponding to the controllers
    :type ctrl_channels: seq<seq<sardana.pool.PoolCounterTimer>>

    :return: a configuration dictionary
    :rtype: dict<>
    '''

    master_ctrl_idx = 0
    master_idx = 0
    configuration = {}
    ctrls_configuration = {}
    configuration['timer'] = timer = ctrl_channels[master_ctrl_idx][master_idx]
    for ctrl, channels in zip(ctrls, ctrl_channels):
        ctrl_data = createConfFromObj(ctrl)
        ctrl_data['channels'] = {}
        for channel in channels:
            channel_conf = createConfFromObj(channel)
            ctrl_data['channels'][channel] = channel_conf
        ctrl_data['timer'] = channels[master_idx]
        ctrls_configuration[ctrl] = ctrl_data
    configuration['controllers'] = ctrls_configuration
    mg_cfg = MeasurementConfiguration()
    mg_cfg._config = configuration
    mg_cfg.hw_sync_monitor = timer
    mg_cfg.hw_sync_timer = timer
    mg_cfg.sw_sync_timer = timer
    mg_cfg.sw_sync_monitor = timer
    mg_cfg.ctrl_hw_sync = ctrls_configuration
    mg_cfg.ctrl_sw_sync = ctrls_configuration

    return mg_cfg


def createMGUserConfiguration(pool, channels):
    '''Method to create MeasurementGroup configuration using strings.


    :param channels: Each tuple: (expchan, associated_trigger, synchronization)
                    First element of the list of lists is the master
                    counter/timer.
                    First element of each list is the master counter/timer
                    from the controller.
    :type channels: seq<seq<tuple(str)>>
    :return: a tuple of three elements: measurement group configuration
             dictionary of strings, sequence of channel ids, sequence of channel
             names
    :rtype: tupe(dict<>, seq<int>, seq<string>
    '''

    channel_ids = []
    channel_names = []
    MG_configuration = {}
    main_master_channel = pool.get_element_by_full_name(channels[0][0][0])
    MG_configuration['timer'] = main_master_channel.full_name
    MG_configuration['monitor'] = main_master_channel.full_name

    all_ctrls_d = {}
    index = 0  # index represents the order of the channels
    for i in range(len(channels)):
        channels_in_ctrl = channels[i]
        master_channel_str = channels[i][0][0]
        master_channel = pool.get_element_by_full_name(master_channel_str)
        ctrl = master_channel.get_controller()
        ctrl_full_name = ctrl.full_name

        ctrl_d = {}
        ctrl_d.update({ctrl_full_name: {}})

        ctrl_data = {}
        ctrl_data['monitor'] = master_channel_str
        ctrl_data['timer'] = master_channel_str
        ctrl_data['synchronization'] = channels[i][0][2]
        ctrl_data['synchronizer'] = channels[i][0][1]
        channels_d = {}
        for chan_idx in range(len(channels_in_ctrl)):
            channel_name_str = channels_in_ctrl[chan_idx][0]
            channel_names.append(channel_name_str)
            channel_element = pool.get_element_by_full_name(channel_name_str)
            channel_ids.append(channel_element.id)
            one_channel_d = {}
            one_channel_d.update({'full_name': channel_name_str})
            one_channel_d.update({'index': index})
            channels_d.update({channel_name_str: one_channel_d})
            index += 1
        ctrl_data['channels'] = {}
        ctrl_data['channels'].update(channels_d)
        ctrl_d[ctrl_full_name] = ctrl_data
        all_ctrls_d.update(ctrl_d)

    MG_configuration.update({'controllers': all_ctrls_d})
    return MG_configuration, channel_ids, channel_names


def createConfbyCtrlKlass(pool, ctrl_klass, ctrl_name):
    pool_mng = pool.get_manager()
    klass = ctrl_klass
    meta = pool_mng.getControllerMetaClass(klass)
    _lib = meta.lib
    # TODO check is is a valid name
    # raise Exception("The %s ctrl exists." % ctrl_name)
    name = full_name = ctrl_name
    ctrl_id = pool.get_free_id()
    lib_name = _lib.full_name
    ctrl_type = ElementType[meta.get_type()]

    cfg = dict({'class_info': None,
                'full_name': full_name,
                'id': ctrl_id,
                'klass': klass,
                'lib_info': None,
                'library': lib_name,
                'name': name,
                'pool': None,
                'properties': {},
                'role_ids': '',
                'type': ctrl_type})
    return cfg


def createCtrlConf(pool, name, klass, lib, props={}):
    cfg = dict({'class_info': None,
                'full_name': None,
                'id': None,
                'klass': None,
                'lib_info': None,
                'library': lib,
                'name': name,
                'pool': None,
                'properties': props,
                'role_ids': '',
                'type': 'ControllerClass'})
    cfg['id'] = pool.get_free_id()
    cfg['name'] = cfg['full_name'] = name
    cfg['klass'] = klass
    cfg['library'] = lib
    return cfg


def createElemConf(pool, axis, name):
    cfg = dict({'axis': None,
                'ctrl': None,
                'full_name': '',
                'id': None,
                'name': '',
                'pool': None})
    cfg['id'] = pool.get_free_id()
    cfg['axis'] = axis
    cfg['name'] = cfg['full_name'] = name
    return cfg


def createConfFromObj(obj):
    cfg = dict({
        'name': obj.name,
        'full_name': obj.full_name,
        'id': obj.id
    })
    # TODO:
    #  enabling the channel - normally done when applying the MG conf.
    #  see poolmeasurementgroup.PoolMeasurementGroup._build_channel_defaults
    if ElementType[obj.get_type()] == 'CTExpChannel':
        cfg['enabled'] = True
    return cfg



#!/usr/bin/env python

##############################################################################
##
# This file is part of Sardana
##
# http://www.sardana-controls.org/
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

__docformat__ = 'restructuredtext'

import sys
import time
from taurus.external.qt import Qt
# TODO: avoid using of PyTango - use Taurus instead
import PyTango
import sardana

from taurus.qt.qtgui.container import TaurusWidget
from taurus.qt.qtgui.display import TaurusLabel
from taurus.qt.qtgui.base import TaurusBaseWidget

from taurus.external.qt import QtCore, QtGui

from taurus.qt.qtcore.communication import SharedDataManager
from taurus.qt.qtgui.input import TaurusValueLineEdit

import taurus.core.util.argparse
import taurus.qt.qtgui.application
from taurus.qt.qtgui.util.ui import UILoadable

from sardana.taurus.qt.qtgui.extra_macroexecutor import TaurusMacroConfigurationDialog

from .selectsignal import SelectSignal


class EngineModesComboBox(Qt.QComboBox, TaurusBaseWidget):
    """ComboBox representing list of engine modes"""

    def __init__(self, parent=None):
        name = self.__class__.__name__
        self.call__init__wo_kw(Qt.QComboBox, parent)
        self.call__init__(TaurusBaseWidget, name)
        self.setSizeAdjustPolicy(Qt.QComboBox.AdjustToContentsOnFirstShow)
        self.setToolTip("Choose a engine mode ...")
        QtCore.QMetaObject.connectSlotsByName(self)

    def loadEngineModeNames(self, enginemodes):
        self.clear()
        self.addItems(enginemodes)


@UILoadable(with_ui="_ui")
class DiffractometerAlignment(TaurusWidget):

    def __init__(self, parent=None, designMode=False):
        TaurusWidget.__init__(self, parent, designMode=designMode)

        self.loadUi(filename="diffractometeralignment.ui")

        self.selectsignal = SelectSignal()

        self._ui.AlignmentStopButton.clicked.connect(self.stop_movements)
        self._ui.AlignmentStoreReflectionButton.clicked.connect(
            self.store_reflection)

        self._ui.MacroServerConnectionButton.clicked.connect(
            self.open_macroserver_connection_panel)

        self._ui.SelectSignalButton.clicked.connect(
            self.open_selectsignal_panel)

        # Create a global SharedDataManager
        Qt.qApp.SDM = SharedDataManager(self)

    @classmethod
    def getQtDesignerPluginInfo(cls):
        ret = TaurusWidget.getQtDesignerPluginInfo()
        ret['module'] = 'diffractometeralignment'
        ret['group'] = 'Taurus Containers'
        ret['container'] = ':/designer/frame.png'
        ret['container'] = False
        return ret

    def setModel(self, model):
        if model is not None:
            self.device = taurus.Device(model)

        self.pseudo_motor_names = []
        for motor in self.device.hklpseudomotorlist:
            self.pseudo_motor_names.append(motor.split(' ')[0])

        self.h_device_name = self.pseudo_motor_names[0]
        self.h_device = taurus.Device(self.h_device_name)
        self.k_device_name = self.pseudo_motor_names[1]
        self.k_device = taurus.Device(self.k_device_name)
        self.l_device_name = self.pseudo_motor_names[2]
        self.l_device = taurus.Device(self.l_device_name)

        # Set model to hkl components

        hmodel = self.h_device_name + "/Position"
        self._ui.taurusValueLineH.setModel(hmodel)
        self._ui.taurusLabelValueH.setModel(hmodel)
        kmodel = self.k_device_name + "/Position"
        self._ui.taurusValueLineK.setModel(kmodel)
        self._ui.taurusLabelValueK.setModel(kmodel)
        lmodel = self.l_device_name + "/Position"
        self._ui.taurusValueLineL.setModel(lmodel)
        self._ui.taurusLabelValueL.setModel(lmodel)

        # Add dynamically the angle widgets

        motor_list = self.device.motorlist
        self.motor_names = []
        self.motor_devices = []

        for motor in self.device.motorlist:
            self.motor_names.append(motor.split(' ')[0])
            self.motor_devices.append(taurus.Device(
                self.motor_names[len(self.motor_names) - 1]))

        self.nb_motors = len(motor_list)

        angles_labels = []
        self.angles_names = []
        angles_taurus_label = []
        angles_taurus_input = []

        gap_x = 650 // self.nb_motors

        try:
            self.angles_names = self.device.motorroles
        except:  # Only for compatibility
            if self.nb_motors == 4:
                self.angles_names.append("omega")
                self.angles_names.append("chi")
                self.angles_names.append("phi")
                self.angles_names.append("theta")
            elif self.nb_motors == 6:
                self.angles_names.append("mu")
                self.angles_names.append("th")
                self.angles_names.append("chi")
                self.angles_names.append("phi")
                self.angles_names.append("gamma")
                self.angles_names.append("delta")

        for i in range(0, self.nb_motors):
            angles_labels.append(QtGui.QLabel(self))
            angles_labels[i].setGeometry(
                QtCore.QRect(150 + gap_x * i, 40, 71, 17))
            angles_labels[i].setLayoutDirection(QtCore.Qt.RightToLeft)
            alname = "angleslabel" + str(i)
            angles_labels[i].setObjectName(alname)
            angles_labels[i].setText(QtGui.QApplication.translate(
                "HKLScan", self.angles_names[i], None))

            angles_taurus_label.append(TaurusLabel(self))
            angles_taurus_label[i].setGeometry(
                QtCore.QRect(150 + gap_x * i, 70, 81, 19))
            atlname = "anglestauruslabel" + str(i)
            angles_taurus_label[i].setObjectName(atlname)
            angles_taurus_label[i].setModel(self.motor_names[i] + "/Position")

            angles_taurus_input.append(TaurusValueLineEdit(self))
            angles_taurus_input[i].setGeometry(
                QtCore.QRect(145 + gap_x * i, 100, 91, 27))
            atlname = "anglestaurusinput" + str(i)
            angles_taurus_input[i].setObjectName(atlname)
            angles_taurus_input[i].setModel(self.motor_names[i] + "/Position")

        # Set model to engine and modes

        enginemodel = model + '/engine'
        self._ui.taurusLabelEngine.setModel(enginemodel)
        enginemodemodel = model + '/enginemode'
        self._ui.taurusLabelEngineMode.setModel(enginemodemodel)

        self.enginemodescombobox = EngineModesComboBox(self)
        self.enginemodescombobox.setGeometry(QtCore.QRect(150, 315, 221, 27))
        self.enginemodescombobox.setObjectName("enginemodeslist")

        self.enginemodescombobox.loadEngineModeNames(self.device.hklmodelist)

        self.enginemodescombobox.currentIndexChanged['QString'].connect(
            self.onModeChanged)

        # Add dynamically the scan buttons, range inputs and 'to max' buttons

        scan_buttons = []
        self.range_inputs = []
        self.tomax_buttons = []  # The text will be change when the max. is computed

        exec_functions = [self.exec_scan1, self.exec_scan2, self.exec_scan3,
                          self.exec_scan4, self.exec_scan5, self.exec_scan6]

        tomax_functions = [self.tomax_scan1, self.tomax_scan2, self.tomax_scan3,
                           self.tomax_scan4, self.tomax_scan5, self.tomax_scan6]

        gap_x = 650 // self.nb_motors

        for i in range(0, self.nb_motors):
            scan_buttons.append(QtGui.QPushButton(self))
            scan_buttons[i].setGeometry(
                QtCore.QRect(150 + gap_x * i, 405, 100, 26))
            wname = "scanbutton" + str(i)
            scan_buttons[i].setObjectName(wname)
            scan_buttons[i].setText(QtGui.QApplication.translate(
                "DiffractometerAlignment", self.angles_names[i], None))
            scan_buttons[i].clicked.connect(exec_functions[i])

            self.range_inputs.append(QtGui.QLineEdit(self))
            self.range_inputs[i].setGeometry(
                QtCore.QRect(150 + gap_x * i, 440, 100, 26))
            self.range_inputs[i].setLayoutDirection(QtCore.Qt.RightToLeft)
            wname = "rangeinput" + str(i)
            self.range_inputs[i].setObjectName(wname)

            self.tomax_buttons.append(QtGui.QPushButton(self))
            self.tomax_buttons[i].setGeometry(
                QtCore.QRect(150 + gap_x * i, 475, 100, 26))
            wname = "tomaxbutton" + str(i)
            self.tomax_buttons[i].setObjectName(wname)
            self.tomax_buttons[i].setText(QtGui.QApplication.translate(
                "DiffractometerAlignment", 'n.n.', None))
            self.tomax_buttons[i].clicked.connect(tomax_functions[i])

    def exec_scan1(self):
        self.exec_scan(0)

    def exec_scan2(self):
        self.exec_scan(1)

    def exec_scan3(self):
        self.exec_scan(2)

    def exec_scan4(self):
        self.exec_scan(3)

    def exec_scan5(self):
        self.exec_scan(4)

    def exec_scan6(self):
        self.exec_scan(5)

    def exec_scan(self, imot):
        macro_command = []

        macro_command.append("_diff_scan")
        macro_command.append(str(self.motor_names[imot]))
        current_pos = self.motor_devices[imot].Position
        range_scan = float(self.range_inputs[imot].text())
        macro_command.append(str(current_pos - range_scan))
        macro_command.append(str(current_pos + range_scan))
        macro_command.append(str(self._ui.NbPointslineEdit.text()))
        macro_command.append(
            str(self.selectsignal._ui.SampleTimelineEdit.text()))
        macro_command.append(str(self.selectsignal._ui.SignallineEdit.text()))

        self.door_device.RunMacro(macro_command)
        while(self.door_device.State()) == PyTango.DevState.RUNNING:
            time.sleep(0.01)
        # TODO: the string parsing should be eliminated and the sardana
        # generic "goto_peak" feature should be used instead - when available
        output_values = self.door_device.read_attribute("Output").value
        if output_values is not None:
            for i in range(len(output_values)):
                if output_values[i] == "Position to move":
                    self.tomax_buttons[imot].setText(QtGui.QApplication.translate(
                        "DiffractometerAlignment", str(output_values[i + 1]),
                        None))

    def tomax_scan1(self):
        self.tomax_scan(0)

    def tomax_scan2(self):
        self.tomax_scan(1)

    def tomax_scan3(self):
        self.tomax_scan(2)

    def tomax_scan4(self):
        self.tomax_scan(3)

    def tomax_scan5(self):
        self.tomax_scan(4)

    def tomax_scan6(self):
        self.tomax_scan(5)

    def tomax_scan(self, imot):
        motor = str(self.motor_names[imot])
        position = str(self.tomax_buttons[imot].text())
        macro_command = ["mv", motor, position]
        self.door_device.RunMacro(macro_command)

    @Qt.pyqtSlot('QString')
    def onModeChanged(self, modename):
        if self.device.engine != "hkl":
            self.device.write_attribute("engine", "hkl")
        self.device.write_attribute("enginemode", str(modename))

    def open_macroserver_connection_panel(self):
        w = TaurusMacroConfigurationDialog(self)
        Qt.qApp.SDM.connectReader("macroserverName", w.selectMacroServer)
        Qt.qApp.SDM.connectReader("doorName", w.selectDoor)
        Qt.qApp.SDM.connectReader("doorName", self.onDoorChanged)
        Qt.qApp.SDM.connectWriter(
            "macroserverName", w, 'macroserverNameChanged')
        Qt.qApp.SDM.connectWriter("doorName", w, 'doorNameChanged')

        w.show()

    def onDoorChanged(self, doorName):
        if doorName != self.door_device_name:
            self.door_device_name = doorName
            self.door_device = taurus.Device(doorName)

    def stop_movements(self):
        self.door_device.StopMacro()

    def store_reflection(self):
        hklref = []
        hklref.append(self.h_device.Position)
        hklref.append(self.k_device.Position)
        hklref.append(self.l_device.Position)

        self.device.write_attribute("addreflection", hklref)

    def open_selectsignal_panel(self):

        self.selectsignal.update_signals(self.door_device_name)
        self.selectsignal.show()


def main():
    parser = taurus.core.util.argparse.get_taurus_parser()
    parser.usage = "%prog <model> [door_name]"
    desc = ("a taurus application for diffractometer alignment: h, k, l " +
            "movements and scans, go to maximum, ...")
    parser.set_description(desc)

    app = taurus.qt.qtgui.application.TaurusApplication(
        cmd_line_parser=parser, app_version=sardana.Release.version)
    app.setApplicationName("diffractometeralignment")
    args = app.get_command_line_args()
    if len(args) < 1:
        msg = "model not set (requires diffractometer controller)"
        parser.error(msg)

    w = DiffractometerAlignment()
    w.model = args[0]
    w.setModel(w.model)

    w.door_device = None
    w.door_device_name = None
    if len(args) > 1:
        w.onDoorChanged(args[1])
    else:
        msg = ("No door name supplied. Connection to MacroServer/Door will " +
               "not not automatically done.")
        app.warning(msg)

    w.show()

    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



#!/usr/bin/env python

##############################################################################
##
# This file is part of Sardana
##
# http://www.sardana-controls.org/
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""Tests for standard macros"""

import unittest
from sardana.macroserver.macros.test import RunMacroTestCase, testRun,\
    getMotors

MOT_NAME1, MOT_NAME2 = getMotors()[:2]

# TODO: these tests randomly causes segfaults. fix it!
# @testRun(macro_name="wu", wait_timeout=1)
# @testRun(macro_name="wa", wait_timeout=1)
# @testRun(macro_name="wa", macro_params=["mot.*"], wait_timeout=1)
# @testRun(macro_name="pwa", wait_timeout=1)
# @testRun(macro_name="wm", macro_params=[MOT_NAME1], wait_timeout=1)
# @testRun(macro_name="wm", macro_params=[MOT_NAME1, MOT_NAME2], wait_timeout=1)
# @testRun(macro_name="wum", macro_params=[MOT_NAME1], wait_timeout=1)
# @testRun(macro_name="wum", macro_params=[MOT_NAME1, MOT_NAME2], wait_timeout=1)
# @testRun(macro_name="pwm", macro_params=[MOT_NAME1], wait_timeout=1)
# @testRun(macro_name="pwm", macro_params=[MOT_NAME1, MOT_NAME2], wait_timeout=1)
# class WhereTest(RunMacroTestCase, unittest.TestCase):
#     """Test case for where macros
#     """
#     pass


@testRun(macro_name="set_lim", macro_params=[MOT_NAME1, "-100", "100"],
         wait_timeout=1)
@testRun(macro_name="set_lm", macro_params=[MOT_NAME1, "-1000", "1000"],
         wait_timeout=1)
class LimTest(RunMacroTestCase, unittest.TestCase):
    """Test case for limit macros
    """
    pass


@testRun(macro_name="set_pos", macro_params=[MOT_NAME1, "0"],
         wait_timeout=1)
@testRun(macro_name="set_user_pos", macro_params=[MOT_NAME1, "0"],
         wait_timeout=1)
class PosTest(RunMacroTestCase, unittest.TestCase):
    """Test case for position macros
    """
    pass


class MoveTest(RunMacroTestCase, unittest.TestCase):
    """Test case for position macros
    """

    def test_move(self):
        self.macro_runs("set_user_pos", macro_params=[MOT_NAME1, "0"],
                        wait_timeout=1)
        self.macro_runs("mv", macro_params=[MOT_NAME1, "1"], wait_timeout=3)
        self.macro_runs("umv", macro_params=[MOT_NAME1, "0"], wait_timeout=3)
        self.macro_runs("mvr", macro_params=[MOT_NAME1, "1"], wait_timeout=3)
        self.macro_runs("umvr", macro_params=[MOT_NAME1, "-1"], wait_timeout=3)


@testRun(macro_params=[MOT_NAME1], wait_timeout=1)
class MstateTest(RunMacroTestCase, unittest.TestCase):

    macro_name = "mstate"


@testRun(macro_params=["blabla"], wait_timeout=1)
class ReportTest(RunMacroTestCase, unittest.TestCase):

    macro_name = "report"



##############################################################################
##
# This file is part of Sardana
##
# http://www.sardana-controls.org/
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This file contains the code for an hypothetical Springfield motor controller
used in documentation"""

import springfieldlib

from sardana import State
from sardana.pool.controller import MotorController


class SpringfieldBaseMotorController(MotorController):
    """The most basic controller intended from demonstration purposes only.
    This is the absolute minimum you have to implement to set a proper motor
    controller able to get a motor position, get a motor state and move a
    motor.

    This example is so basic that it is not even directly described in the
    documentation"""

    MaxDevice = 128

    def __init__(self, inst, props, *args, **kwargs):
        """Constructor"""
        super(SpringfieldBaseMotorController, self).__init__(
            inst, props, *args, **kwargs)
        self.springfield = springfieldlib.SpringfieldMotorHW()

    def ReadOne(self, axis):
        """Get the specified motor position"""
        return self.springfield.getPosition(axis)

    def StateOne(self, axis):
        """Get the specified motor state"""
        springfield = self.springfield
        state = springfield.getState(axis)
        if state == 1:
            return State.On, "Motor is stopped"
        elif state == 2:
            return State.Moving, "Motor is moving"
        elif state == 3:
            return State.Fault, "Motor has an error"

    def StartOne(self, axis, position):
        """Move the specified motor to the specified position"""
        self.springfield.move(axis, position)

    def StopOne(self, axis):
        """Stop the specified motor"""
        self.springfield.stop(axis)


from sardana import DataAccess
from sardana.pool.controller import Type, Description, DefaultValue, Access, FGet, FSet


class SpringfieldMotorController(MotorController):

    axis_attributes = {
        "CloseLoop": {
            Type: bool,
            Description: "(de)activates the motor close loop algorithm",
            DefaultValue: False,
        },
    }

    def getCloseLoop(self, axis):
        return self.springfield.isCloseLoopActive(axis)

    def setCloseLoop(self, axis, value):
        self.springfield.setCloseLoop(axis, value)

    def __init__(self, inst, props, *args, **kwargs):
        super(SpringfieldMotorController, self).__init__(
            inst, props, *args, **kwargs)

        # initialize hardware communication
        self.springfield = springfieldlib.SpringfieldMotorHW()

        # do some initialization
        self._motors = {}

    def AddDevice(self, axis):
        self._motors[axis] = True

    def DeleteDevice(self, axis):
        del self._motors[axis]

    StateMap = {
        1: State.On,
        2: State.Moving,
        3: State.Fault,
    }

    def StateOne(self, axis):
        springfield = self.springfield
        state = self.StateMap[springfield.getState(axis)]
        status = springfield.getStatus(axis)

        limit_switches = MotorController.NoLimitSwitch
        hw_limit_switches = springfield.getLimits(axis)
        if hw_limit_switches[0]:
            limit_switches |= MotorController.HomeLimitSwitch
        if hw_limit_switches[1]:
            limit_switches |= MotorController.UpperLimitSwitch
        if hw_limit_switches[2]:
            limit_switches |= MotorController.LowerLimitSwitch
        return state, status, limit_switches

    def ReadOne(self, axis):
        position = self.springfield.getPosition(axis)
        return position

    def StartOne(self, axis, position):
        self.springfield.move(axis, position)

    def StopOne(self, axis):
        self.springfield.stop(axis)

    def AbortOne(self, axis):
        self.springfield.abort(axis)

    def DefinePosition(self, axis, position):
        self.springfield.setCurrentPosition(axis, position)



#!/usr/bin/env python

##############################################################################
##
# This file is part of Sardana
##
# http://www.sardana-controls.org/
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

import os
import imp
from setuptools import setup, find_packages


def get_release_info():
    name = "release"
    setup_dir = os.path.dirname(os.path.abspath(__file__))
    release_dir = os.path.join(setup_dir, "src", "sardana")
    data = imp.find_module(name, [release_dir])
    release = imp.load_module(name, *data)
    return release

release = get_release_info()

package_dir = {"": "src"}

packages = find_packages(where="src")

provides = [
    'sardana',
    # 'sardana.pool',
    # 'sardana.macroserver',
    # 'sardana.spock',
    # 'sardana.tango',
]

requires = [
    'PyTango (>=9.2.5)',
    'itango (>=0.1.6)',
    'taurus (>= 4.7.0)',
    'lxml (>=2.3)',
]

install_requires = [
    'PyTango>=9.2.5',
    'itango>=0.1.6',
    'taurus>=4.7.0',
    'lxml>=2.3',
    'click',
]


console_scripts = [
    "MacroServer = sardana.tango.macroserver:main",
    "Pool = sardana.tango.pool:main",
    "Sardana = sardana.tango:main",
    "sardanatestsuite = sardana.test.testsuite:main",
    "spock = sardana.spock:main",
]

gui_scripts = [
    "diffractometeralignment = sardana.taurus.qt.qtgui.extra_hkl.diffractometeralignment:main",
    "hklscan = sardana.taurus.qt.qtgui.extra_hkl.hklscan:main",
    "macroexecutor = sardana.taurus.qt.qtgui.extra_macroexecutor.macroexecutor:main",
    "sequencer = sardana.taurus.qt.qtgui.extra_macroexecutor.sequenceeditor:main",
    "ubmatrix = sardana.taurus.qt.qtgui.extra_hkl.ubmatrix:main",
    "showscan = sardana.taurus.qt.qtgui.extra_sardana.showscanonline:main"
]

form_factories = [
    "sardana.pool = sardana.taurus.qt.qtgui.extra_pool.formitemfactory:pool_item_factory"  # noqa
]

entry_points = {
    'console_scripts': console_scripts,
    'gui_scripts': gui_scripts,
    'taurus.form.item_factories': form_factories,
}

classifiers = [
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Environment :: No Input/Output (Daemon)',
    'Environment :: Win32 (MS Windows)',
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
    'Operating System :: Microsoft :: Windows',
    'Operating System :: POSIX',
    'Operating System :: POSIX :: Linux',
    'Operating System :: Unix',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3.5',
    'Topic :: Scientific/Engineering',
    'Topic :: Software Development :: Libraries',
]

setup(name='sardana',
      version=release.version,
      description=release.description,
      long_description=release.long_description,
      author=release.authors['Tiago_et_al'][0],
      maintainer=release.authors['Community'][0],
      maintainer_email=release.authors['Community'][1],
      url=release.url,
      download_url=release.download_url,
      platforms=release.platforms,
      license=release.license,
      keywords=release.keywords,
      packages=packages,
      package_dir=package_dir,
      include_package_data=True,
      classifiers=classifiers,
      entry_points=entry_points,
      provides=provides,
      requires=requires,
      install_requires=install_requires
      )



##############################################################################
##
# This file is part of Sardana
##
# http://www.sardana-controls.org/
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module contains macros that demonstrate the usage of hooks"""

__all__ = ["captain_hook", "captain_hook2", "loop", "hooked_scan",
           "hooked_dummyscan"]

__docformat__ = 'restructuredtext'

from sardana.macroserver.macro import Macro, Type, Hookable, ExecMacroHook


class loop(Macro, Hookable):
    """A macro that executes a for loop. It accepts hooks.
    This macro is part of the examples package. It was written for
    demonstration purposes"""

    hints = {'allowsHooks': ('pre-move', 'post-move', 'pre-acq', 'post-acq')}

    param_def = [['start', Type.Integer, None, 'start point'],
                 ['stop', Type.Integer, None, 'end point'],
                 ['step', Type.Integer, 1, 'step']]

    def run(self, start, stop, step):
        self.info("Starting loop")
        for i in range(start, stop, step):
            self.output("At step %d" % i)
            self.flushOutput()
            for hook, hints in self.hooks:
                self.info("running hook with hints=" + repr(hints))
                hook()
        self.info("Finished loop")


class captain_hook(Macro):
    """A macro that executes a loop macro. A hook was attached so that in each
    step of the loop this hook is executed.
    This macro is part of the examples package. It was written for
    demonstration purposes"""

    param_def = [['start', Type.Integer, None, 'start point'],
                 ['stop', Type.Integer, None, 'end point'],
                 ['step', Type.Integer, 1, 'step']]

    def hook(self):
        self.info("\thook execution")

    def run(self, start, stop, step):
        loop_macro, _ = self.createMacro("loop", start, stop, step)
        loop_macro.hooks = [(self.hook, ["pre-acq"])]
        self.runMacro(loop_macro)


class captain_hook2(Macro):
    """A macro that executes a loop macro. A hook was attached so that in each
    step of the loop this hook is executed.
    This macro is part of the examples package. It was written for
    demonstration purposes"""

    param_def = [['start', Type.Integer, None, 'start point'],
                 ['stop', Type.Integer, None, 'end point'],
                 ['step', Type.Integer, 1, 'step']]

    def hook(self):
        self.execMacroStr(["lsm"])

    def run(self, start, stop, step):
        loop_macro, _ = self.createMacro("loop", start, stop, step)
        #h = self.createExecMacroHook(["lsm"])
        # it gives the "pre-acq" hint to the hook
        loop_macro.hooks = [self.hook]
        self.runMacro(loop_macro)


class hooked_scan(Macro):
    """An example on how to attach hooks to the various hook points of a scan.
    This macro is part of the examples package. It was written for
    demonstration purposes"""

    param_def = [
        ['motor',      Type.Moveable, None, 'Motor to move'],
        ['start_pos',  Type.Float,   None, 'Scan start position'],
        ['final_pos',  Type.Float,   None, 'Scan final position'],
        ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
        ['integ_time', Type.Float,   None, 'Integration time']
    ]

    def hook1(self):
        self.info("\thook1 execution")

    def hook2(self):
        self.info("\thook2 execution")

    def hook3(self):
        self.info("\thook3 execution")

    def hook4(self):
        self.info("\thook4 execution")

    def hook5(self):
        self.info("\thook5 execution")

    def hook6(self):
        self.info("\thook6 execution")

    def run(self, motor, start_pos, final_pos, nr_interv, integ_time):
        ascan, pars = self.createMacro(
            "ascan", motor, start_pos, final_pos, nr_interv, integ_time)
        ascan.hooks = [(self.hook1, ["pre-acq"]),
                       (self.hook2, ["pre-acq", "post-acq",
                                     "pre-move", "post-move", "aaaa"]),
                       self.hook3,
                       (self.hook4, ["pre-scan"]),
                       (self.hook5, ["pre-scan", "post-scan"]),
                       (self.hook6, ["post-step"])]
        self.runMacro(ascan)


class hooked_scan_with_macro(Macro):
    """An example on how to attach macro (in this case without parameters)
    as a hook to the various hook points of a scan.

    This macro is part of the examples package. It was written for
    demonstration purposes"""

    param_def = [
        ['motor', Type.Moveable, None, 'Motor to move'],
        ['start_pos', Type.Float,   None, 'Scan start position'],
        ['final_pos', Type.Float,   None, 'Scan final position'],
        ['nr_interv', Type.Integer, None, 'Number of scan intervals'],
        ['integ_time', Type.Float,   None, 'Integration time'],
        ['macro', Type.Macro, None, 'Macro to be used as hook'],
        ['hook_places', [['hook_place', Type.String, None, 'Hook place']],
            None, 'Hook places where macro will be called']
    ]

    def run(self, motor, start_pos, final_pos, nr_interv, integ_time, macro,
            hook_places):
        ascan, _ = self.createMacro(
            "ascan", motor, start_pos, final_pos, nr_interv, integ_time)
        macro_hook = ExecMacroHook(self, "umv", [["mot01", 1]])
        ascan.hooks = [(macro_hook, hook_places)]
        self.runMacro(ascan)


class hooked_dummyscan(Macro):
    """An example on how to attach hooks to the various hook points of a scan.
    This macro is part of the examples package. It was written for
    demonstration purposes"""

    param_def = [
        ['start_pos',  Type.Float,   None, 'Scan start position'],
        ['final_pos',  Type.Float,   None, 'Scan final position'],
        ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
        ['integ_time', Type.Float,   None, 'Integration time']
    ]

    def hook1(self):
        self.info("\thook1 execution")

    def hook2(self):
        self.info("\thook2 execution")

    def hook3(self):
        self.info("\thook3 execution")

    def run(self, start_pos, final_pos, nr_interv, integ_time):
        dummyscan, pars = self.createMacro(
            "dummyscan", start_pos, final_pos, nr_interv, integ_time)
        dummyscan.hooks = [(self.hook1, ["pre-scan"]), (self.hook2, ["pre-acq",
                                                                     "post-acq", "pre-move", "post-move", "aaaa"]), (self.hook3, ["post-scan"])]
        self.runMacro(dummyscan)



##############################################################################
##
# This file is part of Sardana
##
# http://www.sardana-controls.org/
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Sardana is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Sardana is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This is the standard macro module"""

__all__ = ["ct", "mstate", "mv", "mvr", "pwa", "pwm", "repeat", "set_lim",
           "set_lm", "set_pos", "settimer", "uct", "umv", "umvr", "wa", "wm",
           "tw", "logmacro", "newfile", "plotselect", "pic", "cen",
           "where"]


__docformat__ = 'restructuredtext'

import datetime
import os

import numpy as np
from taurus import Device
from taurus.console.table import Table
import PyTango
from PyTango import DevState

from sardana.macroserver.macro import Macro, macro, Type, ViewOption, \
    iMacro, Hookable
from sardana.macroserver.msexception import StopException, UnknownEnv
from sardana.macroserver.scan.scandata import Record
from sardana.macroserver.macro import Optional

##########################################################################
#
# Motion related macros
#
##########################################################################


class _wm(Macro):
    """Show motor positions"""

    param_def = [
        ['motor_list', [['motor', Type.Moveable, None, 'Motor to move']],
         None, 'List of motor to show'],
    ]

    def run(self, motor_list):
        show_dial = self.getViewOption(ViewOption.ShowDial)
        show_ctrlaxis = self.getViewOption(ViewOption.ShowCtrlAxis)
        pos_format = self.getViewOption(ViewOption.PosFormat)
        motor_width = 9
        motors = {}  # dict(motor name: motor obj)
        requests = {}  # dict(motor name: request id)
        data = {}  # dict(motor name: list of motor data)
        # sending asynchronous requests: neither Taurus nor Sardana extensions
        # allow asynchronous requests - use PyTango asynchronous request model
        for motor in motor_list:
            name = motor.getName()
            motors[name] = motor
            args = ('position',)
            if show_dial:
                args += ('dialposition',)
            _id = motor.read_attributes_asynch(args)
            requests[name] = _id
            motor_width = max(motor_width, len(name))
            data[name] = []
        # get additional motor information (ctrl name & axis)
        if show_ctrlaxis:
            for name, motor in motors.items():
                ctrl_name = self.getController(motor.controller).name
                axis_nb = str(getattr(motor, "axis"))
                data[name].extend((ctrl_name, axis_nb))
                motor_width = max(motor_width, len(ctrl_name), len(axis_nb))
        # collect asynchronous replies
        while len(requests) > 0:
            req2delete = []
            for name, _id in requests.items():
                motor = motors[name]
                try:
                    attrs = motor.read_attributes_reply(_id)
                    for attr in attrs:
                        value = attr.value
                        if value is None:
                            value = float('NaN')
                            if attr.name == 'dialposition':
                                value = motor.getDialPosition()
                                if value is None:
                                    value = float('NaN')
                        data[name].append(value)
                    req2delete.append(name)
                except PyTango.AsynReplyNotArrived:
                    continue
                except PyTango.DevFailed:
                    data[name].append(float('NaN'))
                    if show_dial:
                        data[name].append(float('NaN'))
                    req2delete.append(name)
                    self.debug('Error when reading %s position(s)' % name)
                    self.debug('Details:', exc_info=1)
                    continue
            # removing motors which alredy replied
            for name in req2delete:
                requests.pop(name)
        # define format for numerical values
        fmt = '%c*.%df' % ('%', motor_width - 5)
        if pos_format > -1:
            fmt = '%c*.%df' % ('%', int(pos_format))
        # prepare row headers and formats
        row_headers = []
        t_format = []
        if show_ctrlaxis:
            row_headers += ['Ctrl', 'Axis']
            t_format += ['%*s', '%*s']
        row_headers.append('User')
        t_format.append(fmt)
        if show_dial:
            row_headers.append('Dial')
            t_format.append(fmt)
        # sort the data dict by keys
        col_headers = []
        values = []
        for mot_name, mot_values in sorted(data.items()):
            col_headers.append([mot_name])  # convert name to list
            values.append(mot_values)
        # create and print table
        table = Table(values, elem_fmt=t_format,
                      col_head_str=col_headers, col_head_width=motor_width,
                      row_head_str=row_headers)
        for line in table.genOutput():
            self.output(line)


class _wum(Macro):
    """Show user motor positions"""

    param_def = [
        ['motor_list', [['motor', Type.Moveable, None, 'Motor to move']],
         None, 'List of motor to show'],
    ]

    def prepare(self, motor_list, **opts):
        self.table_opts = {}

    def run(self, motor_list):
        motor_width = 9
        motor_names = []
        motor_pos = []
        motor_list = sorted(motor_list)
        pos_format = self.getViewOption(ViewOption.PosFormat)
        for motor in motor_list:
            name = motor.getName()
            motor_names.append([name])
            pos = motor.getPosition(force=True)
            if pos is None:
                pos = float('NAN')
            motor_pos.append((pos,))
            motor_width = max(motor_width, len(name))

        fmt = '%c*.%df' % ('%', motor_width - 5)
        if pos_format > -1:
            fmt = '%c*.%df' % ('%', int(pos_format))

        table = Table(motor_pos, elem_fmt=[fmt],
                      col_head_str=motor_names, col_head_width=motor_width,
                      **self.table_opts)
        for line in table.genOutput():
            self.output(line)


class wu(Macro):
    """Show all user motor positions"""

    def prepare(self, **opts):
        self.all_motors = self.findObjs('.*', type_class=Type.Moveable)
        self.table_opts = {}

    def run(self):
        nr_motors = len(self.all_motors)
        if nr_motors == 0:
            self.output('No motor defined')
            return

        self.output('Current positions (user) on %s' %
                    datetime.datetime.now().isoformat(' '))
        self.output('')

        self.execMacro('_wum', self.all_motors, **self.table_opts)


class wa(Macro):
    """Show all motor positions"""

    # TODO: duplication of the default value definition is a workaround
    # for #427. See commit message cc3331a for more details.
    param_def = [
        ['filter', [['filter', Type.String, '.*',
                     'a regular expression filter'], {'min': 1}],
         ['.*'], 'a regular expression filter'],
    ]

    def prepare(self, filter, **opts):
        self.all_motors = self.findObjs(filter, type_class=Type.Moveable)
        self.table_opts = {}

    def run(self, filter):
        nr_motors = len(self.all_motors)
        if nr_motors == 0:
            self.output('No motor defined')
            return

        show_dial = self.getViewOption(ViewOption.ShowDial)
        if show_dial:
            self.output('Current positions (user, dial) on %s' %
                        datetime.datetime.now().isoformat(' '))
        else:
            self.output('Current positions (user) on %s' %
                        datetime.datetime.now().isoformat(' '))
        self.output('')
        self.execMacro('_wm', self.all_motors, **self.table_opts)


class pwa(Macro):
    """Show all motor positions in a pretty table"""

    # TODO: duplication of the default value definition is a workaround
    # for #427. See commit message cc3331a for more details.
    param_def = [
        ['filter', [['filter', Type.String, '.*',
                     'a regular expression filter'], {'min': 1}],
         ['.*'], 'a regular expression filter'],
    ]

    def run(self, filter):
        self.execMacro('wa', filter, **Table.PrettyOpts)


class set_lim(Macro):
    """Sets the software limits on the specified motor hello"""
    param_def = [
        ['motor', Type.Moveable, None, 'Motor name'],
        ['low',   Type.Float, None, 'lower limit'],
        ['high',   Type.Float, None, 'upper limit']
    ]

    def run(self, motor, low, high):
        name = motor.getName()
        self.debug("Setting user limits for %s" % name)
        motor.getPositionObj().setLimits(low, high)
        self.output("%s limits set to %.4f %.4f (user units)" %
                    (name, low, high))


class set_lm(Macro):
    """Sets the dial limits on the specified motor"""
    param_def = [
        ['motor', Type.Motor, None, 'Motor name'],
        ['low',   Type.Float, None, 'lower limit'],
        ['high',   Type.Float, None, 'upper limit']
    ]

    def run(self, motor, low, high):
        name = motor.getName()
        self.debug("Setting dial limits for %s" % name)
        motor.getDialPositionObj().setLimits(low, high)
        self.output("%s limits set to %.4f %.4f (dial units)" %
                    (name, low, high))


class set_pos(Macro):
    """Sets the position of the motor to the specified value"""

    param_def = [
        ['motor', Type.Motor, None, 'Motor name'],
        ['pos',   Type.Float, None, 'Position to move to']
    ]

    def run(self, motor, pos):
        name = motor.getName()
        old_pos = motor.getPosition(force=True)
        motor.definePosition(pos)
        self.output("%s reset from %.4f to %.4f" % (name, old_pos, pos))


class set_user_pos(Macro):
    """Sets the USER position of the motor to the specified value (by
    changing OFFSET and keeping DIAL)"""

    param_def = [
        ['motor', Type.Motor, None, 'Motor name'],
        ['pos',   Type.Float, None, 'Position to move to']
    ]

    def run(self, motor, pos):
        name = motor.getName()
        old_pos = motor.getPosition(force=True)
        offset_attr = motor.getAttribute('Offset')
        old_offset = offset_attr.read().rvalue.magnitude
        new_offset = pos - (old_pos - old_offset)
        offset_attr.write(new_offset)
        msg = "%s reset from %.4f (offset %.4f) to %.4f (offset %.4f)" % (
            name, old_pos, old_offset, pos, new_offset)
        self.output(msg)


class wm(Macro):
    """Show the position of the specified motors."""

    param_def = [
        ['motor_list', [['motor', Type.Moveable, None,
                         'Motor to see where it is']],
         None, 'List of motor to show'],
    ]

    def prepare(self, motor_list, **opts):
        self.table_opts = {}

    def run(self, motor_list):
        motor_width = 10
        motor_names = []
        motor_pos = []

        show_dial = self.getViewOption(ViewOption.ShowDial)
        show_ctrlaxis = self.getViewOption(ViewOption.ShowCtrlAxis)
        pos_format = self.getViewOption(ViewOption.PosFormat)

        for motor in motor_list:

            max_len = 0
            if show_ctrlaxis:
                axis_nb = getattr(motor, "axis")
                ctrl_name = self.getController(motor.controller).name
                max_len = max(max_len, len(ctrl_name), len(str(axis_nb)))
            name = motor.getName()
            max_len = max(max_len, len(name))

            max_len = max_len + 5
            if max_len < 14:
                max_len = 14  # Length of 'Not specified'

            str_fmt = "%c%ds" % ('%', int(max_len))

            name = str_fmt % name

            motor_names.append([name])
            posObj = motor.getPositionObj()
            if pos_format > -1:
                fmt = '%c.%df' % ('%', int(pos_format))

            try:
                val1 = fmt % motor.getPosition(force=True)
                val1 = str_fmt % val1
            except:
                val1 = str_fmt % motor.getPosition(force=True)

            val2 = str_fmt % posObj.getMaxRange().magnitude
            val3 = str_fmt % posObj.getMinRange().magnitude

            if show_ctrlaxis:
                valctrl = str_fmt % (ctrl_name)
                valaxis = str_fmt % str(axis_nb)
                upos = list(map(str, [valctrl, valaxis, ' ', val2, val1,
                                      val3]))
            else:
                upos = list(map(str, ['', val2, val1, val3]))
            pos_data = upos
            if show_dial:
                try:
                    val1 = fmt % motor.getDialPosition(force=True)
                    val1 = str_fmt % val1
                except Exception:
                    val1 = str_fmt % motor.getDialPosition(force=True)

                dPosObj = motor.getDialPositionObj()
                val2 = str_fmt % dPosObj.getMaxRange().magnitude
                val3 = str_fmt % dPosObj.getMinRange().magnitude

                dpos = list(map(str, [val2, val1, val3]))
                pos_data += [''] + dpos

            motor_pos.append(pos_data)

        elem_fmt = (['%*s'] + ['%*s'] * 5) * 2
        row_head_str = []
        if show_ctrlaxis:
            row_head_str += ['Ctrl', 'Axis']
        row_head_str += ['User', ' High', ' Current', ' Low']
        if show_dial:
            row_head_str += ['Dial', ' High', ' Current', ' Low']
        table = Table(motor_pos, elem_fmt=elem_fmt, row_head_str=row_head_str,
                      col_head_str=motor_names, col_head_width=motor_width,
                      **self.table_opts)
        for line in table.genOutput():
            self.output(line)


class wum(Macro):
    """Show the user position of the specified motors."""

    param_def = [
        ['motor_list', [['motor', Type.Moveable, None,
                         'Motor to see where it is']],
         None, 'List of motor to show'],
    ]

    def prepare(self, motor_list, **opts):
        self.table_opts = {}

    def run(self, motor_list):
        motor_width = 10
        motor_names = []
        motor_pos = []

        for motor in motor_list:
            name = motor.getName()
            motor_names.append([name])
            posObj = motor.getPositionObj()
            upos = list(map(str, [posObj.getMaxRange().magnitude,
                                  motor.getPosition(force=True),
                                  posObj.getMinRange().magnitude]))
            pos_data = [''] + upos

            motor_pos.append(pos_data)

        elem_fmt = (['%*s'] + ['%*s'] * 3) * 2
        row_head_str = ['User', ' High', ' Current', ' Low', ]
        table = Table(motor_pos, elem_fmt=elem_fmt, row_head_str=row_head_str,
                      col_head_str=motor_names, col_head_width=motor_width,
                      **self.table_opts)
        for line in table.genOutput():
            self.output(line)


class pwm(Macro):
    """Show the position of the specified motors in a pretty table"""

    param_def = [
        ['motor_list', [['motor', Type.Moveable, None, 'Motor to move']],
         None, 'List of motor to show'],
    ]

    def run(self, motor_list):
        self.execMacro('wm', motor_list, **Table.PrettyOpts)


class mv(Macro):
    """Move motor(s) to the specified position(s)"""

    param_def = [
        ['motor_pos_list',
         [['motor', Type.Moveable, None, 'Motor to move'],
          ['pos',   Type.Float, None, 'Position to move to']],
         None, 'List of motor/position pairs'],
    ]

    def run(self, motor_pos_list):
        motors, positions = [], []
        for m, p in motor_pos_list:
            motors.append(m)
            positions.append(p)
            self.debug("Starting %s movement to %s", m.getName(), p)
        motion = self.getMotion(motors)
        state, pos = motion.move(positions)
        if state != DevState.ON:
            self.warning("Motion ended in %s", state.name)
            msg = []
            for motor in motors:
                msg.append(motor.information())
            self.info("\n".join(msg))


class mstate(Macro):
    """Prints the state of a motor"""

    param_def = [['motor', Type.Moveable, None, 'Motor to check state']]

    def run(self, motor):
        self.info("Motor %s" % str(motor.stateObj.read().rvalue))


class umv(Macro):
    """Move motor(s) to the specified position(s) and update"""

    param_def = mv.param_def

    def prepare(self, motor_pos_list, **opts):
        self.all_names = []
        self.all_pos = []
        self.print_pos = False
        for motor, pos in motor_pos_list:
            self.all_names.append([motor.getName()])
            pos, posObj = motor.getPosition(force=True), motor.getPositionObj()
            self.all_pos.append([pos])
            posObj.subscribeEvent(self.positionChanged, motor)

    def run(self, motor_pos_list):
        self.print_pos = True
        try:
            self.execMacro('mv', motor_pos_list)
        finally:
            self.finish()

    def finish(self):
        self._clean()
        self.printAllPos()

    def _clean(self):
        for motor, pos in self.getParameters()[0]:
            posObj = motor.getPositionObj()
            try:
                posObj.unsubscribeEvent(self.positionChanged, motor)
            except Exception as e:
                print(str(e))
                raise e

    def positionChanged(self, motor, position):
        idx = self.all_names.index([motor.getName()])
        self.all_pos[idx] = [position]
        if self.print_pos:
            self.printAllPos()

    def printAllPos(self):
        motor_width = 10
        table = Table(self.all_pos, elem_fmt=['%*.4f'],
                      col_head_str=self.all_names, col_head_width=motor_width)
        self.outputBlock(table.genOutput())
        self.flushOutput()


class mvr(Macro):
    """Move motor(s) relative to the current position(s)"""

    param_def = [
        ['motor_disp_list',
         [['motor', Type.Moveable, None, 'Motor to move'],
          ['disp',  Type.Float, None, 'Relative displacement']],
         None, 'List of motor/displacement pairs'],
    ]

    def run(self, motor_disp_list):
        motor_pos_list = []
        for motor, disp in motor_disp_list:
            pos = motor.getPosition(force=True)
            if pos is None:
                self.error("Cannot get %s position" % motor.getName())
                return
            else:
                pos += disp
            motor_pos_list.append([motor, pos])
        self.execMacro('mv', motor_pos_list)


class umvr(Macro):
    """Move motor(s) relative to the current position(s) and update"""

    param_def = mvr.param_def

    def run(self, motor_disp_list):
        motor_pos_list = []
        for motor, disp in motor_disp_list:
            pos = motor.getPosition(force=True)
            if pos is None:
                self.error("Cannot get %s position" % motor.getName())
                return
            else:
                pos += disp
            motor_pos_list.append([motor, pos])
        self.execMacro('umv', motor_pos_list)

# TODO: implement tw macro with param repeats in order to be able to pass
# multiple motors and multiple deltas. Also allow to pass the integration time
# in order to execute the measurement group acquisition after each move and
# print the results. Basically follow the SPEC's API:
# https://certif.com/spec_help/tw.html


class tw(iMacro):
    """Tweak motor by variable delta"""

    param_def = [
        ['motor', Type.Moveable, "test", 'Motor to move'],
        ['delta',   Type.Float, None, 'Amount to tweak']
    ]

    def run(self, motor, delta):
        self.output(
            "Indicate direction with + (or p) or - (or n) or enter")
        self.output(
            "new step size. Type something else (or ctrl-C) to quit.")
        self.output("")
        if np.sign(delta) == -1:
            a = "-"
        if np.sign(delta) == 1:
            a = "+"
        while a in ('+', '-', 'p', 'n'):
            pos = motor.position
            a = self.input("%s = %s, which way? " % (
                motor, pos), default_value=a, data_type=Type.String)
            try:
                # check if the input is a new delta
                delta = float(a)
                # obtain the sign of the new delta
                if np.sign(delta) == -1:
                    a = "-"
                else:
                    a = "+"
            except:
                # convert to the common sign
                if a == "p":
                    a = "+"
                # convert to the common sign
                elif a == "n":
                    a = "-"
                # the sign is already correct, just continue
                elif a in ("+", "-"):
                    pass
                else:
                    msg = "Typing '%s' caused 'tw' macro to stop." % a
                    self.info(msg)
                    raise StopException()
                # invert the delta if necessary
                if (a == "+" and np.sign(delta) < 0) or \
                   (a == "-" and np.sign(delta) > 0):
                    delta = -delta
            pos += delta
            self.mv(motor, pos)


##########################################################################
#
# Data acquisition related macros
#
##########################################################################


def _value_to_repr(data):
    if data is None:
        return "<nodata>"
    elif np.ndim(data) > 0:
        return list(np.shape(data))
    else:
        return data


class _ct:

    def dump_information(self, elements):
        msg = ["Elements ended acquisition with:"]
        for element in elements:
            msg.append(element.information())
        self.info("\n".join(msg))


class ct(Macro, Hookable, _ct):
    """Count for the specified time on the measurement group
       or experimental channel given as second argument
       (if not given the active measurement group is used)"""

    hints = {'allowsHooks': ('pre-acq', 'post-acq')}
    param_def = [
        ['integ_time', Type.Float, 1.0, 'Integration time'],
        ['countable_elem', Type.Countable, Optional,
         'Countable element e.g. MeasurementGroup or ExpChannel']
    ]

    def prepare(self, integ_time, countable_elem, **opts):
        if countable_elem is None:
            try:
                self.countable_elem_name = self.getEnv('ActiveMntGrp')
            except UnknownEnv:
                return
            self.countable_elem = self.getObj(
                self.countable_elem_name, type_class=Type.MeasurementGroup)
        else:
            self.countable_elem_name = countable_elem.name
            self.countable_elem = countable_elem

    def run(self, integ_time, countable_elem):
        if self.countable_elem is None:
            msg = ('Unknown countable {0} element. Use macro parameter or'
                   'ActiveMntGrp environment variable'.format(
                                                    self.countable_elem_name))
            self.error(msg)
            return
        # integration time has to be accessible from with in the hooks
        # so declare it also instance attribute
        self.integ_time = integ_time
        self.debug("Counting for %s sec", integ_time)
        self.outputDate()
        self.output('')
        self.flushOutput()

        for preAcqHook in self.getHooks('pre-acq'):
            preAcqHook()

        try:
            state, data = self.countable_elem.count(integ_time)
        except Exception:
            if self.countable_elem.type == Type.MeasurementGroup:
                names = self.countable_elem.ElementList
                elements = [self.getObj(name) for name in names]
                self.dump_information(elements)
            raise
        if state != DevState.ON:
            if self.countable_elem.type == Type.MeasurementGroup:
                names = self.countable_elem.ElementList
                elements = [self.getObj(name) for name in names]
                self.dump_information(elements)
                raise ValueError("Acquisition ended with {}".format(
                    state.name.capitalize()))

        for postAcqHook in self.getHooks('post-acq'):
            postAcqHook()

        names, counts = [], []
        if self.countable_elem.type == Type.MeasurementGroup:
            meas_grp = self.countable_elem
            for ch_info in meas_grp.getChannelsEnabledInfo():
                names.append('  %s' % ch_info.label)
                ch_data = data.get(ch_info.full_name)
                counts.append(_value_to_repr(ch_data))
        else:
            channel = self.countable_elem
            names.append("  %s" % channel.name)
            counts.append(_value_to_repr(data))
            # to be compatible with measurement group count
            data = {channel.full_name: data}
        self.setData(Record(data))
        table = Table([counts], row_head_str=names, row_head_fmt='%*s',
                      col_sep='  =  ')
        for line in table.genOutput():
            self.output(line)


class uct(Macro, _ct):
    """Count on the active measurement group and update"""

    param_def = [
        ['integ_time', Type.Float, 1.0, 'Integration time'],
        ['countable_elem', Type.Countable, Optional,
         'Countable element e.g. MeasurementGroup or ExpChannel']
    ]

    def prepare(self, integ_time, countable_elem, **opts):

        self.print_value = False

        if countable_elem is None:
            try:
                self.countable_elem_name = self.getEnv('ActiveMntGrp')
            except UnknownEnv:
                return
            self.countable_elem = self.getObj(self.countable_elem_name)
        else:
            self.countable_elem_name = countable_elem.name
            self.countable_elem = countable_elem

        if self.countable_elem is None:
            return

        self.channels = []
        self.values = []
        if self.countable_elem.type == Type.MeasurementGroup:
            names = self.countable_elem.getChannelLabels()
            self.names = [[n] for n in names]
            for channel_info in self.countable_elem.getChannels():
                full_name = channel_info["full_name"]
                channel = Device(full_name)
                self.channels.append(channel)
                value = channel.getValue(force=True)
                self.values.append([value])
                valueObj = channel.getValueObj_()
                valueObj.subscribeEvent(self.counterChanged, channel)
        else:
            channel = self.countable_elem
            self.names = [[channel.getName()]]
            channel = Device(channel.full_name)
            self.channels.append(channel)
            value = channel.getValue(force=True)
            self.values.append([value])
            valueObj = channel.getValueObj_()
            valueObj.subscribeEvent(self.counterChanged, channel)

    def run(self, integ_time, countable_elem):
        if self.countable_elem is None:
            msg = ('Unknown countable {0} element. Use macro parameter or'
                   'ActiveMntGrp environment variable'.format(
                                                    self.countable_elem_name))
            self.error(msg)
            return

        self.print_value = True
        try:
            state, data = self.countable_elem.count(integ_time)
        except Exception:
            if self.countable_elem.type == Type.MeasurementGroup:
                names = self.countable_elem.ElementList
                elements = [self.getObj(name) for name in names]
                self.dump_information(elements)
            raise
        finally:
            self.finish()
        if state != DevState.ON:
            if self.countable_elem.type == Type.MeasurementGroup:
                names = self.countable_elem.ElementList
                elements = [self.getObj(name) for name in names]
                self.dump_information(elements)
                raise ValueError("Acquisition ended with {}".format(
                    state.name.capitalize()))
        self.setData(Record(data))
        self.printAllValues()

    def finish(self):
        self._clean()

    def _clean(self):
        for channel in self.channels:
            valueObj = channel.getValueObj_()
            valueObj.unsubscribeEvent(self.counterChanged, channel)

    def counterChanged(self, channel, value):
        idx = self.names.index([channel.getName()])
        self.values[idx] = [value]
        if self.print_value and not self.isStopped():
            self.printAllValues()

    def printAllValues(self):
        ch_width = 10
        table = Table(self.values, elem_fmt=['%*.4f'], col_head_str=self.names,
                      col_head_width=ch_width)
        self.outputBlock(table.genOutput())
        self.flushOutput()


class settimer(Macro):
    """Defines the timer channel for the active measurement group"""

    env = ('ActiveMntGrp',)

    param_def = [
        ['timer', Type.ExpChannel,   None, 'Timer'],
    ]

    def run(self, timer):
        mnt_grp_name = self.getEnv('ActiveMntGrp')
        mnt_grp = self.getObj(mnt_grp_name, type_class=Type.MeasurementGroup)

        if mnt_grp is None:
            self.error('ActiveMntGrp is not defined or has invalid value.\n'
                       'please define a valid active measurement group '
                       'before setting a timer')
            return

        try:
            mnt_grp.getConfiguration().setTimer(timer.getName())
        except Exception as e:
            self.output(str(e))
            self.output(
                "%s is not a valid channel in the active measurement group"
                % timer)


@macro([['message', [['message_item', Type.String, None,
                      'message item to be reported']], None,
         'message to be reported']])
def report(self, message):
    """Logs a new record into the message report system (if active)"""
    self.report(' '.join(message))


class logmacro(Macro):
    """ Turn on/off logging of the spock output.

    .. note::
        The logmacro class has been included in Sardana
        on a provisional basis. Backwards incompatible changes
        (up to and including its removal) may occur if
        deemed necessary by the core developers
    """

    param_def = [
        ['offon', Type.Boolean, None, 'Unset/Set logging'],
        ['mode', Type.Integer, -1, 'Mode: 0 append, 1 new file'],
    ]

    def run(self, offon, mode):
        if offon:
            if mode == 1:
                self.setEnv('LogMacroMode', True)
            elif mode == 0:
                self.setEnv('LogMacroMode', False)
            self.setEnv('LogMacro', True)
        else:
            self.setEnv('LogMacro', False)


class repeat(Hookable, Macro):
    """This macro executes as many repetitions of a set of macros as
    specified by nr parameter. The macros to be repeated can be
    given as parameters or as body hooks.
    If both are given first will be executed the ones given as
    parameters and then the ones given as body hooks.
    If nr has negative value, repetitions will be executed until you
    stop repeat macro.

    .. note::
        The repeat macro has been included in Sardana
        on a provisional basis. Backwards incompatible changes
        (up to and including removal of the macro) may occur if
        deemed necessary by the core developers."""

    hints = {'allowsHooks': ('body',)}

    param_def = [
        ['nr', Type.Integer, None, 'Nr of iterations'],
        ['macro_name_params', [
                ['token', Type.String,
                 None, 'Macro name and parameters (if any)'],
                {'min': 0}
            ],
            None, "List with macro name and parameters (if any)"]
    ]

    def prepare(self, nr, macro_name_params):
        self.bodyHooks = self.getHooks("body")
        self.macro_name_params = macro_name_params

    def __loop(self):
        self.checkPoint()
        if len(self.macro_name_params) > 0:
            for macro_cmd in self.macro_name_params:
                self.execMacro(macro_cmd)
        for bodyHook in self.bodyHooks:
            bodyHook()

    def run(self, nr, macro_name_params):
        if nr < 0:
            while True:
                self.__loop()
        else:
            for i in range(nr):
                self.__loop()
                progress = ((i + 1) / nr) * 100
                yield progress


class newfile(Hookable, Macro):
    """ Sets the ScanDir and ScanFile as well as ScanID in the environment.

    If ScanFilePath is only a file name, the ScanDir must be set externally
    via `senv ScanDir <PathToScanFile>` or using the %expconf. Otherwise,
    the path in ScanFilePath must be absolute and existing on the
    MacroServer host.

    The ScanID should be set to the value before the upcoming scan number.
    Default value is 0.
    """

    hints = {'allowsHooks': ('post-newfile')}

    param_def = [
        ['ScanFilePath_list',
         [['ScanFilePath', Type.String, None, '(ScanDir/)ScanFile']],
         None, 'List of (ScanDir/)ScanFile'],
        ['ScanID', Type.Integer, 0, 'Scan ID'],
    ]

    def run(self, ScanFilePath_list, ScanID):
        path_list = []
        fileName_list = []
        # traverse the repeat parameters for the ScanFilePath_list
        for i, ScanFilePath in enumerate(ScanFilePath_list):
            path = os.path.dirname(ScanFilePath)
            fileName = os.path.basename(ScanFilePath)
            if not path and i == 0:
                # first entry and no given ScanDir: check if ScanDir exists
                try:
                    ScanDir = self.getEnv('ScanDir')
                except UnknownEnv:
                    ScanDir = ''
                if not (isinstance(ScanDir, str) and len(ScanDir) > 0):
                    msg = ('Data is not stored until ScanDir is correctly '
                           'set! Provide ScanDir with newfile macro: '
                           '`newfile [<ScanDir>/<ScanFile>] <ScanID>` '
                           'or `senv ScanDir <ScanDir>` or with %expconf')
                    self.error(msg)
                    return
                else:
                    path = ScanDir
            elif not path and i > 0:
                # not first entry and no given path: use path of last iteration
                path = path_list[i-1]
            elif not os.path.isabs(path):
                # relative path
                self.error('Only absolute path are allowed!')
                return
            else:
                # absolute path
                path = os.path.normpath(path)

            if i > 0 and (path not in path_list):
                # check if paths are equal
                self.error('Multiple paths to the data files are not allowed')
                return
            elif not os.path.exists(path):
                # check if folder exists
                self.error('Path %s does not exists on the host of the '
                           'MacroServer and has to be created in '
                           'advance.' % path)
                return
            else:
                self.debug('Path %s appended.' % path)
                path_list.append(path)

            if not fileName:
                self.error('No filename is given.')
                return
            elif fileName in fileName_list:
                self.error('Duplicate filename %s is not allowed.' % fileName)
                return
            else:
                self.debug('Filename is %s.' % fileName)
                fileName_list.append(fileName)

        if ScanID < 1:
            ScanID = 0

        self.setEnv('ScanFile', fileName_list)
        self.setEnv('ScanDir', path_list[0])
        self.setEnv('ScanID', ScanID)

        self.output('ScanDir is\t: %s', path_list[0])
        for i, ScanFile in enumerate(fileName_list):
            if i == 0:
                self.output('ScanFile set to\t: %s', ScanFile)
            else:
                self.output('\t\t  %s', ScanFile)
        self.output('Next scan is\t: #%d', ScanID+1)

        for postNewfileHook in self.getHooks('post-newfile'):
            postNewfileHook()


class plotselect(Macro):
    """select channels for plotting in the active measurement group"""

    env = ("ActiveMntGrp", )
    param_def = [
          ['channel',
           [['channel', Type.ExpChannel, 'None', ""], {'min': 0}],
           None,
           "List of channels to plot"],
     ]

    def run(self, channel):
        active_meas_grp = self.getEnv('ActiveMntGrp')
        meas_grp = self.getMeasurementGroup(active_meas_grp)
        self.output("Active measurement group: {}".format(meas_grp.name))

        plot_channels_ok = []
        enabled_channels = meas_grp.getEnabled()
        # check channels first
        for chan in channel:
            enabled = enabled_channels.get(chan.name)
            if enabled is None:
                self.warning("{} not in {}".format(chan.name, meas_grp.name))
            else:
                plot_channels_ok.append(chan.name)
                if not enabled:
                    self.warning("{} is disabled".format(chan.name))
                else:
                    self.output("{} selected for plotting".format(chan.name))
        # set the plot type and plot axis in the meas_group
        meas_grp.setPlotType("No", apply=False)
        meas_grp.setPlotType("Spectrum", *plot_channels_ok, apply=False)
        meas_grp.setPlotAxes(["<mov>"], *plot_channels_ok)


class _movetostatspos(Macro):
    """This macro does the logic for pic and cen"""

    env = ("ScanStats", )

    param_def = [
        ['channel', Type.ExpChannel, Optional, 'name of channel'],
        ['caller', Type.String, None, 'caller (pic or cen)']
    ]

    def run(self, channel, caller):
        stats = self.getEnv('ScanStats', door_name=self.getDoorName())

        if channel is None:
            # use first channel in stats
            channel = next(iter(stats['Stats']))
        else:
            if channel.name in stats['Stats']:
                channel = channel.name
            else:
                raise Exception("channel {} not present in ScanStats".format(
                                channel.name))

        if caller == 'pic':
            stats_value = 'maxpos'
            stats_str = 'PIC'
        elif caller == 'cen':
            stats_value = 'cen'
            stats_str = 'CEN'
        else:
            raise Exception("caller {} is unknown".format(caller))

        motor_name = stats['Motor']
        motor = self.getMotion([motor_name])
        current_pos = motor.readPosition()[0]
        pos = stats['Stats'][channel][stats_value]

        self.info("move motor {:s} from current position\nat {:.4f}\n"
                  "to {:s} of counter {:s}\nat {:.4f}".format(motor_name,
                                                              current_pos,
                                                              stats_str,
                                                              channel,
                                                              pos))
        motor.move(pos)


class pic(Macro):
    """This macro moves the motor of the last scan to the PEAK position for a
    given channel. If no channel is given, it selects the first channel from
    the ScanStats env variable.
    """

    param_def = [
        ['channel', Type.ExpChannel, Optional, 'name of channel']
    ]

    def run(self, channel):
        self.execMacro('_movetostatspos', channel, 'pic')


class cen(Macro):
    """This macro moves the motor of the last scan to the CEN position for a
    given channel. If no channel is given, it selects the first channel from
    the ScanStats env variable.
    """

    param_def = [
        ['channel', Type.ExpChannel, Optional, 'name of channel']
    ]

    def run(self, channel):
        self.execMacro('_movetostatspos', channel, 'cen')


class where(Macro):
    """This macro shows the current position of the last scanned motor."""

    env = ("ScanStats", )

    def run(self):
        motor_name = self.getEnv('ScanStats')['Motor']
        motor = self.getMoveable(motor_name)
        self.info("motor {:s} is\nat {:.4f}".format(motor_name,
                                                    motor.getPosition()))



import os
import sys
from PyQt4 import QtGui, QtCore, Qt


class SardanaBasePage(QtGui.QWizardPage):

    def __init__(self, parent=None):
        QtGui.QWizardPage.__init__(self, parent)
        self._item_funcs = {}

        self.setPixmap(QtGui.QWizard.LogoPixmap, QtGui.QPixmap(":/logo.jpg"))

        layout = QtGui.QVBoxLayout()

        self._panel = QtGui.QWidget()

        self._status_label = QtGui.QLabel()
        self._status_label.setAutoFillBackground(True)
        palette = self._status_label.palette()
        gradient = QtGui.QLinearGradient(0, 0, 0, 15)
        gradient.setColorAt(0.0, Qt.QColor.fromRgb(60, 150, 255))
        gradient.setColorAt(0.5, Qt.QColor.fromRgb(0,  85, 227))
        gradient.setColorAt(1.0, Qt.QColor.fromRgb(60, 150, 255))
        gradient.setSpread(QtGui.QGradient.RepeatSpread)
        palette.setBrush(QtGui.QPalette.Window, QtGui.QBrush(gradient))
        palette.setBrush(QtGui.QPalette.WindowText, Qt.Qt.white)

        layout.addWidget(self._panel)
        layout.addStretch()
        layout.addWidget(self._status_label)
        self.setLayout(layout)

    def __setitem__(self, name, value):
        self._item_funcs[name] = value

    def __getitem__(self, name):
        return self._item_funcs[name]

    def setStatus(self, text):
        self._status_label.setText(text)

    def getPanelWidget(self):
        return self._panel


class SardanaIntroBasePage(QtGui.QWizardPage):

    def __init__(self, parent=None):
        QtGui.QWizardPage.__init__(self, parent)

        self.setTitle('Introduction')
        self.setPixmap(QtGui.QWizard.WatermarkPixmap,
                       QtGui.QPixmap(":/watermark.jpg"))

        label = QtGui.QLabel(self.getIntroText())
        label.setWordWrap(True)
        layout = QtGui.QVBoxLayout()
        layout.addWidget(label)
        self.setLayout(layout)

    def getIntroText(self):
        return """This wizard will guide you through the Sardana configuration"""


class SardanaBaseWizard(QtGui.QWizard):

    def __init__(self, parent=None):
        QtGui.QWizard.__init__(self, parent)
        self._item_funcs = {}
        self._pages = {}

    def __setitem__(self, name, value):
        self._item_funcs[name] = value

    def __getitem__(self, name):
        for id in self.getPages():
            p = self.page(id)
            if isinstance(p, SardanaBasePage):
                try:
                    return p[name]()
                except Exception, e:
                    pass
        return self._item_funcs[name]()
        return None

    def addPage(self, page):
        id = QtGui.QWizard.addPage(self, page)
        self._pages[id] = page

    def setPage(self, id, page):
        QtGui.QWizard.setPage(self, id, page)
        self._pages[id] = page

    def getPages(self):
        return self._pages


def get_resources():
    res_fname = os.path.abspath(__file__)
    res_fname = os.path.splitext(res_fname)[0] + '.rcc'
    return res_fname


def t1():
    app = QtGui.QApplication([])
    QtCore.QResource.registerResource(get_resources())
    w = QtGui.QWizard()
    intro = SardanaBaseIntroPage()
    base1 = SardanaBasePage()
    base1.setTitle('Empty title')
    base1.setSubTitle('Empty sub-title')
    base1.setStatus("The status bar")
    w.addPage(intro)
    w.addPage(base1)
    w.show()
    sys.exit(app.exec_())


def main():
    t1()

if __name__ == "__main__":
    main()



class BaseNode:
    """BaseNode, stores reference to data."""

    def __init__(self, data):
        self.data = data


class BranchNode(BaseNode):
    """BranchNode, apart of reference to data, stores a list of
    children Nodes."""

    def __init__(self, data):
        BaseNode.__init__(self, data)
        self.children = []

    def addChild(self, child):
        self.children.append(child)


class LeafNode(BaseNode):
    """LeafMode, just stores reference to data."""

    def __init__(self, data):
        BaseNode.__init__(self, data)


class Tree:
    """Base tree class, stores reference to root Node object"""

    def __init__(self, root):
        self._root = root

    def root(self):
        return self._root
